@@ -26,7 +26,6 @@
 import os
 import re
 
-from gi.repository import Gdk
 from gi.repository import Gtk
 
 from pynicotine.gtkgui.dialogs import option_dialog
@@ -141,18 +140,17 @@ class Downloads(TransferList):
             re.compile(outfilter)
             self.frame.np.config.sections["transfers"]["downloadregexp"] = outfilter
             # Send error messages for each failed filter to log window
-            if len(failed) >= 1:
+            if failed:
                 errors = ""
 
                 for dfilter, error in failed.items():
                     errors += "Filter: %s Error: %s " % (dfilter, error)
 
-                error = _("Error: %(num)d Download filters failed! %(error)s ", {'num': len(failed), 'error': errors})
-                log.add(error)
+                log.add(_("Error: %(num)d Download filters failed! %(error)s "), {'num': len(failed), 'error': errors})
 
         except Exception as e:
             # Strange that individual filters _and_ the composite filter both fail
-            log.add(_("Error: Download Filter failed! Verify your filters. Reason: %s", e))
+            log.add(_("Error: Download Filter failed! Verify your filters. Reason: %s"), e)
             self.frame.np.config.sections["transfers"]["downloadregexp"] = ""
 
     def selected_results_all_data(self, model, path, iterator, data):
@@ -232,28 +230,7 @@ class Downloads(TransferList):
         command = self.frame.np.config.sections["ui"]["filemanager"]
         open_file_path(final_path, command)
 
-    def on_key_press_event(self, widget, event):
-
-        key = Gdk.keyval_name(event.keyval)
-
-        self.select_transfers()
-
-        if key in ("T", "t"):
-            self.on_abort_transfer(widget)
-        elif key in ("R", "r"):
-            self.on_retry_transfer(widget)
-        elif key in ("C", "c") and event.state in (Gdk.ModifierType.CONTROL_MASK, Gdk.ModifierType.LOCK_MASK | Gdk.ModifierType.CONTROL_MASK):
-            self.on_copy_file_path(widget)
-        elif key == "Delete":
-            self.on_abort_transfer(widget, clear=True)
-        else:
-            # No key match, continue event
-            return False
-
-        widget.stop_emission_by_name("key_press_event")
-        return True
-
-    def _on_play_files(self, widget, prefix=""):
+    def on_play_files(self, widget, prefix=""):
 
         downloaddir = self.frame.np.config.sections["transfers"]["downloaddir"]
 
@@ -331,17 +308,3 @@ class Downloads(TransferList):
 
         self.popup_menu.popup()
         return True
-
-    def on_abort_transfer(self, widget, clear=False):
-        self.select_transfers()
-        self.abort_transfers(clear)
-
-    def on_clear_queued(self, widget):
-        self.clear_transfers(["Queued"])
-
-    def on_retry_transfer(self, widget):
-
-        self.select_transfers()
-
-        for transfer in self.selected_transfers:
-            self.frame.np.transfers.retry_download(transfer)
