@@ -35,7 +35,6 @@ from gi.repository import Gio
 from gi.repository import GLib
 from gi.repository import Gtk
 
-import _thread
 from pynicotine import slskmessages
 from pynicotine import slskproto
 from pynicotine.gtkgui import utils
@@ -82,18 +81,20 @@ from pynicotine.utils import version
 
 class NicotineFrame:
 
-    def __init__(self, application, data_dir, config, plugins, use_trayicon, start_hidden, bindip=None, port=None):
+    def __init__(self, application, data_dir, config, plugins, use_trayicon, start_hidden, ci_mode, bindip=None, port=None):
+
+        if not ci_mode:
+            # Show errors in the GUI from here on
+            sys.excepthook = self.on_critical_error
 
         self.application = application
         self.clip = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
         self.clip_data = ""
         self.data_dir = data_dir
         self.gui_dir = os.path.dirname(os.path.realpath(__file__))
+        self.ci_mode = ci_mode
         self.current_tab_label = None
         self.checking_update = False
-        self.rescanning = False
-        self.brescanning = False
-        self.needrescan = False
         self.autoaway = False
         self.awaytimerid = None
         self.shutdown = False
@@ -349,14 +350,8 @@ class NicotineFrame:
 
         """ Scanning """
 
-        # Slight delay to prevent minor performance hit when compressing large file share
-        timer = threading.Timer(2.0, self.rescan_startup)
-        timer.setName("RescanSharesTimer")
-        timer.setDaemon(True)
-        timer.start()
-
         # Deactivate public shares related menu entries if we don't use them
-        if config["transfers"]["friendsonly"] or not config["transfers"]["shared"]:
+        if config["transfers"]["friendsonly"]:
             self.rescan_public_action.set_enabled(False)
             self.browse_public_shares_action.set_enabled(False)
 
@@ -438,10 +433,14 @@ class NicotineFrame:
         self.uploads.init_interface(self.np.transfers.uploads)
         self.downloads.init_interface(self.np.transfers.downloads)
 
+        self.privatechats.login()
+        self.userbrowse.login()
+        self.userinfo.login()
+
         if msg.banner != "":
             log.add(msg.banner)
 
-        return self.privatechats, self.chatrooms, self.userinfo, self.userbrowse, self.searches, self.downloads, self.uploads, self.userlist, self.interests
+        return self.privatechats, self.chatrooms, self.userinfo, self.userbrowse, self.downloads, self.uploads, self.userlist, self.interests
 
     def init_spell_checker(self):
 
@@ -502,7 +501,10 @@ class NicotineFrame:
                             loaded = True
 
                         except Exception as e:
-                            log.add(_("Error loading custom icon %(path)s: %(error)s") % {"path": path, "error": str(e)})
+                            log.add(_("Error loading custom icon %(path)s: %(error)s"), {
+                                "path": path,
+                                "error": str(e)
+                            })
 
                 if name not in self.images:
                     self.images[name] = self.load_ui_icon(name)
@@ -619,6 +621,16 @@ class NicotineFrame:
         for widget in self.__dict__.values():
             update_widget_visuals(widget)
 
+    """ General """
+
+    def show_info_message(self, title, message):
+
+        dialog = Gtk.MessageDialog(type=Gtk.MessageType.WARNING, buttons=Gtk.ButtonsType.OK, message_format=title)
+        dialog.set_transient_for(self.application.get_active_window())
+        dialog.format_secondary_text(message)
+        dialog.run()
+        dialog.destroy()
+
     """ Connection """
 
     def on_network_event(self, msgs):
@@ -633,7 +645,7 @@ class NicotineFrame:
 
     def network_callback(self, msgs):
 
-        if len(msgs) > 0:
+        if msgs:
             GLib.idle_add(self.on_network_event, msgs)
 
     def conn_close(self, conn, addr):
@@ -896,14 +908,14 @@ class NicotineFrame:
             return
 
         # Clear any potential messages queued up to this point (should not happen)
-        while not self.np.queue.empty():
-            self.np.queue.get(0)
+        while self.np.queue:
+            self.np.queue.popleft()
 
         self.set_user_status("...")
 
         server = self.np.config.sections["server"]["server"]
         self.set_status_text(_("Connecting to %(host)s:%(port)s"), {'host': server[0], 'port': server[1]})
-        self.np.queue.put(slskmessages.ServerConn(None, server))
+        self.np.queue.append(slskmessages.ServerConn(None, server))
 
         if self.np.servertimer is not None:
             self.np.servertimer.cancel()
@@ -912,7 +924,7 @@ class NicotineFrame:
     def on_disconnect(self, *args):
         self.disconnect_action.set_enabled(False)
         self.np.manualdisconnect = True
-        self.np.queue.put(slskmessages.ConnClose(self.np.active_server_conn))
+        self.np.queue.append(slskmessages.ConnClose(self.np.active_server_conn))
 
     def on_away(self, *args):
         self.away = not self.away
@@ -928,12 +940,12 @@ class NicotineFrame:
 
         self.tray.set_away(self.away)
 
-        self.np.queue.put(slskmessages.SetStatus(self.away and 1 or 2))
+        self.np.queue.append(slskmessages.SetStatus(self.away and 1 or 2))
         self.away_action.set_state(GLib.Variant.new_boolean(self.away))
         self.privatechats.update_visuals()
 
     def on_check_privileges(self, *args):
-        self.np.queue.put(slskmessages.CheckPrivileges())
+        self.np.queue.append(slskmessages.CheckPrivileges())
 
     def on_get_privileges(self, *args):
         import urllib.parse
@@ -1112,27 +1124,9 @@ class NicotineFrame:
         self.on_settings(page='Shares')
 
     def on_rescan(self, *args, rebuild=False):
-
-        if self.rescanning:
-            return
-
-        self.rescanning = True
-
-        self.rescan_public_action.set_enabled(False)
-        self.browse_public_shares_action.set_enabled(False)
-
         self.np.shares.rescan_public_shares(rebuild)
 
     def on_buddy_rescan(self, *args, rebuild=False):
-
-        if self.brescanning:
-            return
-
-        self.brescanning = True
-
-        self.rescan_buddy_action.set_enabled(False)
-        self.browse_buddy_shares_action.set_enabled(False)
-
         self.np.shares.rescan_buddy_shares(rebuild)
 
     def on_browse_public_shares(self, *args, folder=None):
@@ -1146,7 +1140,11 @@ class NicotineFrame:
         else:
             msg = self.np.shares.compressed_shares_normal
 
-        _thread.start_new_thread(self.parse_local_shares, (login, msg, folder, "normal"))
+        thread = threading.Thread(target=self.parse_local_shares, args=(login, msg, folder, "normal"))
+        thread.name = "LocalShareParser"
+        thread.daemon = True
+        thread.start()
+
         self.userbrowse.show_user(login, indeterminate_progress=True)
 
     def on_browse_buddy_shares(self, *args, folder=None):
@@ -1160,7 +1158,11 @@ class NicotineFrame:
         else:
             msg = self.np.shares.compressed_shares_buddy
 
-        _thread.start_new_thread(self.parse_local_shares, (login, msg, folder, "buddy"))
+        thread = threading.Thread(target=self.parse_local_shares, args=(login, msg, folder, "buddy"))
+        thread.name = "LocalBuddyShareParser"
+        thread.daemon = True
+        thread.start()
+
         self.userbrowse.show_user(login, indeterminate_progress=True)
 
     # Modes
@@ -1209,7 +1211,11 @@ class NicotineFrame:
     def on_check_latest(self, *args):
 
         if not self.checking_update:
-            _thread.start_new_thread(self._on_check_latest, ())
+            thread = threading.Thread(target=self._on_check_latest)
+            thread.name = "UpdateChecker"
+            thread.daemon = True
+            thread.start()
+
             self.checking_update = True
 
     def _on_check_latest(self):
@@ -1740,57 +1746,6 @@ class NicotineFrame:
         else:
             self.show_tab(tab_box)
 
-    """ Scanning """
-
-    def rescan_startup(self):
-
-        if self.rescanning:
-            return
-
-        if self.np.config.sections["transfers"]["rescanonstartup"]:
-
-            # Rescan public shares if needed
-            if not self.np.config.sections["transfers"]["friendsonly"] and \
-                    self.np.config.sections["transfers"]["shared"]:
-                GLib.idle_add(self.on_rescan)
-
-            # Rescan buddy shares if needed
-            if self.np.config.sections["transfers"]["enablebuddyshares"]:
-                GLib.idle_add(self.on_buddy_rescan)
-
-        else:
-            if not self.np.config.sections["transfers"]["friendsonly"]:
-                self.np.shares.compress_shares("normal")
-
-            if self.np.config.sections["transfers"]["enablebuddyshares"]:
-                self.np.shares.compress_shares("buddy")
-
-    def rescan_finished(self, type):
-
-        if type == "buddy":
-            GLib.idle_add(self._buddy_rescan_finished)
-
-        elif type == "normal":
-            GLib.idle_add(self._rescan_finished)
-
-    def _buddy_rescan_finished(self):
-
-        if self.np.config.sections["transfers"]["enablebuddyshares"]:
-            self.rescan_buddy_action.set_enabled(True)
-            self.browse_buddy_shares_action.set_enabled(True)
-
-        self.brescanning = False
-        self.BuddySharesProgress.hide()
-
-    def _rescan_finished(self):
-
-        if self.np.config.sections["transfers"]["shared"]:
-            self.rescan_public_action.set_enabled(True)
-            self.browse_public_shares_action.set_enabled(True)
-
-        self.rescanning = False
-        self.SharesProgress.hide()
-
     """ Transfer Statistics """
 
     def update_stat_value(self, stat_id, stat_value):
@@ -1798,6 +1753,12 @@ class NicotineFrame:
 
     """ Search """
 
+    def set_wishlist_interval(self, msg):
+        self.searches.set_wishlist_interval(msg)
+
+    def show_search_result(self, msg, username, country):
+        self.searches.show_search_result(msg, username, country)
+
     def on_settings_searches(self, widget):
         self.on_settings(page='Searches')
 
@@ -1868,7 +1829,7 @@ class NicotineFrame:
             if self.np.transfers is not None:
 
                 msg.totalupl = self.np.transfers.get_total_uploads_allowed()
-                msg.queuesize = self.np.transfers.get_upload_queue_sizes()[0]
+                msg.queuesize = self.np.transfers.get_upload_queue_size()
                 msg.slotsavail = self.np.transfers.allow_new_uploads()
                 ua = self.np.config.sections["transfers"]["remotedownloads"]
                 if ua:
@@ -1969,7 +1930,7 @@ class NicotineFrame:
 
         return mode
 
-    """ Private Chat """
+    """ Chat """
 
     def on_settings_logging(self, widget):
         self.on_settings(page='Logging')
@@ -1984,80 +1945,12 @@ class NicotineFrame:
         self.privatechats.send_message(text, show_user=True)
         clear_entry(widget)
 
-    """ Chat """
-
-    def auto_replace(self, message):
-        if self.np.config.sections["words"]["replacewords"]:
-            autoreplaced = self.np.config.sections["words"]["autoreplaced"]
-            for word, replacement in autoreplaced.items():
-                message = message.replace(str(word), str(replacement))
-
-        return message
-
-    def censor_chat(self, message):
-        if self.np.config.sections["words"]["censorwords"]:
-            filler = self.np.config.sections["words"]["censorfill"]
-            censored = self.np.config.sections["words"]["censored"]
-            for word in censored:
-                word = str(word)
-                message = message.replace(word, filler * len(word))
-
-        return message
-
-    def entry_completion_find_match(self, completion, entry_text, iterator, widget):
-        model = completion.get_model()
-        item_text = model.get_value(iterator, 0)
-        ix = widget.get_position()
-        config = self.np.config.sections["words"]
-
-        if entry_text is None or entry_text == "" or entry_text.isspace() or item_text is None:
-            return False
-
-        # Get word to the left of current position
-        if " " in entry_text:
-            split_key = entry_text[:ix].split(" ")[-1]
-        else:
-            split_key = entry_text
-
-        if split_key.isspace() or split_key == "" or len(split_key) < config["characters"]:
-            return False
-
-        # case-insensitive matching
-        if item_text.lower().startswith(split_key) and item_text.lower() != split_key:
-            return True
-
-        return False
-
-    def entry_completion_found_match(self, completion, model, iterator, widget):
-        current_text = widget.get_text()
-        ix = widget.get_position()
-        # if more than a word has been typed, we throw away the
-        # one to the left of our current position because we want
-        # to replace it with the matching word
-
-        if " " in current_text:
-            prefix = " ".join(current_text[:ix].split(" ")[:-1])
-            suffix = " ".join(current_text[ix:].split(" "))
-
-            # add the matching word
-            new_text = "%s %s%s" % (prefix, model[iterator][0], suffix)
-            # set back the whole text
-            widget.set_text(new_text)
-            # move the cursor at the end
-            widget.set_position(len(prefix) + len(model[iterator][0]) + 1)
-        else:
-            new_text = "%s" % (model[iterator][0])
-            widget.set_text(new_text)
-            widget.set_position(-1)
-        # stop the event propagation
-        return True
-
     def on_create_room(self, widget, *args):
 
         room = widget.get_text()
         private = self.RoomType.get_active()
 
-        self.np.queue.put(slskmessages.JoinRoom(room, private))
+        self.np.queue.append(slskmessages.JoinRoom(room, private))
         widget.set_text("")
 
     def on_show_chat_buttons(self, widget=None):
@@ -2112,165 +2005,8 @@ class NicotineFrame:
     def on_settings_ban_ignore(self, widget):
         self.on_settings(page='BanList')
 
-    def ban_user(self, user):
-        if self.np.transfers is not None:
-            self.np.transfers.ban_user(user)
-
-    def user_ip_is_blocked(self, user):
-        for ip, username in self.np.config.sections["server"]["ipblocklist"].items():
-            if user == username:
-                return True
-        return False
-
-    def blocked_user_ip(self, user):
-        for ip, username in self.np.config.sections["server"]["ipblocklist"].items():
-            if user == username:
-                return ip
-        return None
-
-    def user_ip_is_ignored(self, user):
-        for ip, username in self.np.config.sections["server"]["ipignorelist"].items():
-            if user == username:
-                return True
-        return False
-
-    def ignored_user_ip(self, user):
-        for ip, username in self.np.config.sections["server"]["ipignorelist"].items():
-            if user == username:
-                return ip
-        return None
-
-    def ignore_ip(self, ip):
-        if ip is None or ip == "" or ip.count(".") != 3:
-            return
-
-        ipignorelist = self.np.config.sections["server"]["ipignorelist"]
-
-        if ip not in ipignorelist:
-            ipignorelist[ip] = ""
-            self.np.config.write_configuration()
-
-            if self.settingswindow is not None:
-                self.settingswindow.pages["IgnoreList"].set_settings(self.np.config.sections)
-
-    def on_ignore_ip(self, user):
-        if user not in self.np.users or not isinstance(self.np.users[user].addr, tuple):
-            if user not in self.np.ipignore_requested:
-                self.np.ipignore_requested[user] = 0
-
-            self.np.queue.put(slskmessages.GetPeerAddress(user))
-            return
-
-        ipignorelist = self.np.config.sections["server"]["ipignorelist"]
-        ip, port = self.np.users[user].addr
-
-        if ip not in ipignorelist or self.np.config.sections["server"]["ipignorelist"][ip] != user:
-            self.np.config.sections["server"]["ipignorelist"][ip] = user
-            self.np.config.write_configuration()
-
-            if self.settingswindow is not None:
-                self.settingswindow.pages["IgnoreList"].set_settings(self.np.config.sections)
-
-    def on_un_ignore_ip(self, user):
-        ipignorelist = self.np.config.sections["server"]["ipignorelist"]
-
-        if self.user_ip_is_ignored(user):
-            ip = self.ignored_user_ip(user)
-
-            if ip is not None:
-                del ipignorelist[ip]
-                self.np.config.write_configuration()
-
-                if self.settingswindow is not None:
-                    self.settingswindow.pages["IgnoreList"].set_settings(self.np.config.sections)
-                return True
-
-        if user not in self.np.users:
-            if user not in self.np.ipignore_requested:
-                self.np.ipignore_requested[user] = 1
-            self.np.queue.put(slskmessages.GetPeerAddress(user))
-            return
-
-        if not isinstance(self.np.users[user].addr, tuple):
-            return
-
-        ip, port = self.np.users[user].addr
-        if ip in ipignorelist:
-            del ipignorelist[ip]
-            self.np.config.write_configuration()
-
-            if self.settingswindow is not None:
-                self.settingswindow.pages["IgnoreList"].set_settings(self.np.config.sections)
-
-    def on_block_user(self, user):
-        if user not in self.np.users or not isinstance(self.np.users[user].addr, tuple):
-            if user not in self.np.ipblock_requested:
-                self.np.ipblock_requested[user] = 0
-
-            self.np.queue.put(slskmessages.GetPeerAddress(user))
-            return
-
-        ip, port = self.np.users[user].addr
-        if ip not in self.np.config.sections["server"]["ipblocklist"] or self.np.config.sections["server"]["ipblocklist"][ip] != user:
-            self.np.config.sections["server"]["ipblocklist"][ip] = user
-            self.np.config.write_configuration()
-
-            if self.settingswindow is not None:
-                self.settingswindow.pages["BanList"].set_settings(self.np.config.sections)
-
-    def on_un_block_user(self, user):
-        if self.user_ip_is_blocked(user):
-            ip = self.blocked_user_ip(user)
-
-            if ip is not None:
-                del self.np.config.sections["server"]["ipblocklist"][ip]
-                self.np.config.write_configuration()
-
-                if self.settingswindow is not None:
-                    self.settingswindow.pages["BanList"].set_settings(self.np.config.sections)
-                return True
-
-        if user not in self.np.users:
-            if user not in self.np.ipblock_requested:
-                self.np.ipblock_requested[user] = 1
-
-            self.np.queue.put(slskmessages.GetPeerAddress(user))
-            return
-
-        if not isinstance(self.np.users[user].addr, tuple):
-            return
-
-        ip, port = self.np.users[user].addr
-        if ip in self.np.config.sections["server"]["ipblocklist"]:
-            del self.np.config.sections["server"]["ipblocklist"][ip]
-            self.np.config.write_configuration()
-
-            if self.settingswindow is not None:
-                self.settingswindow.pages["BanList"].set_settings(self.np.config.sections)
-
-    def unban_user(self, user):
-        if user in self.np.config.sections["server"]["banlist"]:
-            self.np.config.sections["server"]["banlist"].remove(user)
-            self.np.config.write_configuration()
-
-    def ignore_user(self, user):
-        if user not in self.np.config.sections["server"]["ignorelist"]:
-            self.np.config.sections["server"]["ignorelist"].append(user)
-            self.np.config.write_configuration()
-
-    def unignore_user(self, user):
-        if user in self.np.config.sections["server"]["ignorelist"]:
-            self.np.config.sections["server"]["ignorelist"].remove(user)
-            self.np.config.write_configuration()
-
     """ Various """
 
-    def popup_message(self, popup):
-        dialog = Gtk.MessageDialog(type=Gtk.MessageType.WARNING, buttons=Gtk.ButtonsType.OK, message_format=popup.title)
-        dialog.format_secondary_text(popup.message)
-        dialog.connect('response', lambda dialog, response: dialog.destroy())
-        dialog.show()
-
     def focus_combobox(self, button):
 
         # We have the button of a combobox, find the entry
@@ -2479,7 +2215,11 @@ class NicotineFrame:
             GLib.idle_add(self.BuddySharesProgress.set_fraction, value)
 
     def set_scan_indeterminate(self, sharestype):
-        _thread.start_new_thread(self._set_scan_indeterminate, (sharestype,))
+
+        thread = threading.Thread(target=self._set_scan_indeterminate, args=(sharestype,))
+        thread.name = "ScanProgressBar"
+        thread.daemon = True
+        thread.start()
 
     def _set_scan_indeterminate(self, sharestype):
 
@@ -2550,15 +2290,18 @@ class NicotineFrame:
 
         config = self.np.config.sections
 
-        self.np.update_debug_log_options()
+        should_log = config["logging"]["debug_file_output"]
+        log_folder = config["logging"]["debuglogsdir"]
+        timestamp_format = config["logging"]["log_timestamp"]
+
+        log.update_debug_log_options(should_log, log_folder, timestamp_format)
 
         # UPnP
-        if (not config["server"]["upnp"] or needportmap) and self.np.upnp_timer:
+        if not config["server"]["upnp"] and self.np.upnp_timer:
             self.np.upnp_timer.cancel()
 
         if needportmap:
-            self.np.upnp_interval = config["server"]["upnp_interval"]
-            _thread.start_new_thread(self.np.add_upnp_portmapping, ())
+            self.np.add_upnp_portmapping()
 
         # Download/upload limits
         if self.np.transfers:
@@ -2634,12 +2377,7 @@ class NicotineFrame:
         if self.np.transfers is not None:
             self.np.transfers.check_upload_queue()
 
-        if needrescan:
-            self.needrescan = True
-
-        if msg == "ok" and self.needrescan:
-
-            self.needrescan = False
+        if msg == "ok" and needrescan:
 
             # Rescan public shares if needed
             if not self.np.config.sections["transfers"]["friendsonly"]:
@@ -2664,6 +2402,33 @@ class NicotineFrame:
 
     """ Exit """
 
+    def on_critical_error(self, exc_type, value, tb):
+
+        from traceback import format_tb
+
+        option_dialog(
+            parent=self.application.get_active_window(),
+            title=_("Critical Error"),
+            message=_("Nicotine+ has encountered a critical error and needs to exit. Please copy the following error and include it in a bug report:") +
+            "\n\nType: %s\nValue: %s\nTraceback: %s" % (exc_type, value, ''.join(format_tb(tb))),
+            third=_("Report Bug"),
+            cancel=False,
+            callback=self.on_critical_error_response
+        )
+
+        raise value
+
+    def on_critical_error_response(self, dialog, response, data):
+
+        if response == Gtk.ResponseType.OK:
+
+            dialog.destroy()
+            self.on_quit()
+
+        elif response == Gtk.ResponseType.REJECT:
+
+            self.on_report_bug()
+
     def on_delete_event(self, widget, event):
 
         if not self.np.config.sections["ui"]["exitdialog"]:
@@ -2758,7 +2523,7 @@ class NicotineFrame:
 
 class MainApp(Gtk.Application):
 
-    def __init__(self, data_dir, config, plugins, trayicon, start_hidden, bindip, port):
+    def __init__(self, data_dir, config, plugins, trayicon, start_hidden, ci_mode, bindip, port):
 
         application_id = "org.nicotine_plus.Nicotine"
 
@@ -2771,6 +2536,7 @@ class MainApp(Gtk.Application):
         self.plugins = plugins
         self.trayicon = trayicon
         self.start_hidden = start_hidden
+        self.ci_mode = ci_mode
         self.bindip = bindip
         self.port = port
 
@@ -2785,6 +2551,7 @@ class MainApp(Gtk.Application):
                 self.plugins,
                 self.trayicon,
                 self.start_hidden,
+                self.ci_mode,
                 self.bindip,
                 self.port
             )
