@@ -23,7 +23,6 @@
 
 import operator
 import os
-import random
 import re
 import sre_constants
 
@@ -42,6 +41,7 @@ from pynicotine.gtkgui.utils import human_size
 from pynicotine.gtkgui.utils import human_speed
 from pynicotine.gtkgui.utils import IconNotebook
 from pynicotine.gtkgui.utils import initialise_columns
+from pynicotine.gtkgui.utils import keyval_to_hardware_keycode
 from pynicotine.gtkgui.utils import load_ui_elements
 from pynicotine.gtkgui.utils import PopupMenu
 from pynicotine.gtkgui.utils import save_columns
@@ -63,7 +63,6 @@ class Searches(IconNotebook):
 
         self.frame = frame
 
-        self.searchid = int(random.random() * (2 ** 31 - 1))
         self.searches = {}
         self.usersearches = {}
         self.users = {}
@@ -159,101 +158,34 @@ class Searches(IconNotebook):
                 (users, text) = feedback
 
         else:
-            log.add_warning(_("Unknown search mode, not using plugin system. Fix me!"))
+            log.add_warning("Unknown search mode, not using plugin system. Fix me!")
             feedback = True
 
-        if feedback is not None:
-            self.do_search(text, search_mode, users, room)
+        if feedback is None:
+            return
 
-    def do_search(self, text, mode, users=[], room=None):
+        search_id, searchterm_with_excluded, searchterm_without_excluded = self.frame.np.search.do_search(text, search_mode, users, room)
 
-        # Get excluded words (starting with "-")
-        searchterm_words = text.split()
-        searchterm_words_ignore = (p[1:] for p in searchterm_words if p.startswith('-') and len(p) > 1)
-
-        # Remove words starting with "-", results containing these are excluded by us later
-        searchterm_without_excluded = re.sub(r'(\s)-\w+', r'\1', text)
-
-        if self.frame.np.config.sections["searches"]["remove_special_chars"]:
-            """
-            Remove special characters from search term
-            SoulseekQt doesn't seem to send search results if special characters are included (July 7, 2020)
-            """
-            searchterm_without_excluded = re.sub(r'\W+', ' ', searchterm_without_excluded)
-
-        # Remove trailing whitespace
-        searchterm_without_excluded = searchterm_without_excluded.strip()
-
-        # Append excluded words
-        searchterm_with_excluded = searchterm_without_excluded
-
-        for word in searchterm_words_ignore:
-            searchterm_with_excluded += " -" + word
-
-        items = self.frame.np.config.sections["searches"]["history"]
-
-        if searchterm_with_excluded in items:
-            items.remove(searchterm_with_excluded)
-
-        items.insert(0, searchterm_with_excluded)
-
-        # Clear old items
-        del items[15:]
-        self.frame.np.config.write_configuration()
+        search = self.create_tab(search_id, searchterm_with_excluded, search_mode, showtab=True)
+        if search["tab"] is not None:
+            self.set_current_page(self.page_num(search["tab"].Main))
 
         # Repopulate the combo list
         self.frame.SearchCombo.remove_all()
 
+        items = self.frame.np.config.sections["searches"]["history"]
+
         for i in items:
             if not isinstance(i, str):
                 continue
 
             self.frame.SearchCombo.append_text(i)
 
-        if mode == "user" and users != [] and users[0] != '':
-            self.usersearches[self.searchid] = users
+        if search_mode == "user" and users:
+            self.usersearches[search_id] = users
 
-        search = self.create_tab(self.searchid, searchterm_with_excluded, mode, showtab=True)
-        if search["tab"] is not None:
-            self.set_current_page(self.page_num(search["tab"].Main))
-
-        if mode == "global":
-            self.do_global_search(self.searchid, searchterm_without_excluded)
-
-        elif mode == "rooms":
-            self.do_rooms_search(self.searchid, searchterm_without_excluded, room)
-
-        elif mode == "buddies":
-            self.do_buddies_search(self.searchid, searchterm_without_excluded)
-
-        elif mode == "user" and users != [] and users[0] != '':
-            self.do_peer_search(self.searchid, searchterm_without_excluded, users)
-
-        self.searchid += 1
-
-    def do_global_search(self, id, text):
-        self.frame.np.queue.put(slskmessages.FileSearch(id, text))
-
-        """ Request a list of related searches from the server.
-        Seemingly non-functional since 2018 (always receiving empty lists). """
-
-        # self.frame.np.queue.put(slskmessages.RelatedSearch(text))
-
-    def do_rooms_search(self, id, text, room=None):
-        if room is not None:
-            self.frame.np.queue.put(slskmessages.RoomSearch(room, id, text))
-        else:
-            for room in self.frame.chatrooms.joinedrooms:
-                self.frame.np.queue.put(slskmessages.RoomSearch(room, id, text))
-
-    def do_buddies_search(self, id, text):
-        for i in self.frame.np.config.sections["server"]["userlist"]:
-            user = i[0]
-            self.frame.np.queue.put(slskmessages.UserSearch(user, id, text))
-
-    def do_peer_search(self, id, text, users):
-        for user in users:
-            self.frame.np.send_message_to_peer(user, slskmessages.FileSearchRequest(None, id, text))
+    def set_wishlist_interval(self, msg):
+        self.wish_list.set_interval(msg)
 
     def clear_search_history(self):
 
@@ -332,7 +264,7 @@ class Searches(IconNotebook):
         label = fulltext[:length]
         self.append_page(tab.Main, label, tab.on_close, fulltext=fulltext)
 
-    def show_result(self, msg, username, country):
+    def show_search_result(self, msg, username, country):
 
         try:
             search = self.searches[msg.token]
@@ -419,38 +351,13 @@ class Searches(IconNotebook):
             ("#" + _("Copy Search Term"), self.searches[search_id]["tab"].on_copy_search_term),
             ("", None),
             ("#" + _("Clear All Results"), self.searches[search_id]["tab"].on_clear),
-            ("#" + _("Close All Tabs"), menu.on_close_all_tabs, self),
+            ("#" + _("Close All Tabs"), self.searches[search_id]["tab"].on_close_all_tabs),
             ("#" + _("_Close Tab"), self.searches[search_id]["tab"].on_close)
         )
 
         menu.popup()
         return True
 
-    def on_tab_click(self, widget, event, child):
-
-        search_id = self.get_search_id(child)
-
-        if search_id is None:
-            log.add_warning(_("Search ID was none when clicking tab"))
-            return False
-
-        if triggers_context_menu(event):
-            return self.on_tab_popup(widget, child)
-
-        if event.button == 2:
-            self.searches[search_id]["tab"].on_close(widget)
-            return True
-
-        return False
-
-    def close_all_tabs(self, dialog, response, data):
-
-        if response == Gtk.ResponseType.OK:
-            for search_id in self.searches.copy():
-                self.searches[search_id]["tab"].on_close(dialog)
-
-        dialog.destroy()
-
 
 class Search:
 
@@ -518,7 +425,7 @@ class Search:
             str                   # (18) color
         )
 
-        self.column_numbers = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18]
+        self.column_numbers = list(range(self.resultsmodel.get_n_columns()))
         color_col = 18
         self.cols = cols = initialise_columns(
             "file_search",
@@ -587,7 +494,7 @@ class Search:
 
     def on_tooltip(self, widget, x, y, keyboard_mode, tooltip):
 
-        country_tooltip = show_country_tooltip(widget, x, y, tooltip, 13, stripprefix='')
+        country_tooltip = show_country_tooltip(widget, x, y, tooltip, 13, strip_prefix="")
         file_path_tooltip = show_file_path_tooltip(widget, x, y, tooltip, 12)
 
         if country_tooltip:
@@ -610,12 +517,12 @@ class Search:
             self.FilterOutEntry.set_text(str(sfilter[1]))
             self.FilterSizeEntry.set_text(str(sfilter[2]))
             self.FilterBitrateEntry.set_text(str(sfilter[3]))
-            self.FilterFreeSlot.set_active(str(sfilter[4]))
+            self.FilterFreeSlot.set_active(sfilter[4])
 
-            if(len(sfilter) > 5):
+            if len(sfilter) > 5:
                 self.FilterCountryEntry.set_text(str(sfilter[5]))
 
-            if(len(sfilter) > 6):
+            if len(sfilter) > 6:
                 self.FilterTypeEntry.set_text(str(sfilter[6]))
 
             self.on_refilter(None)
@@ -1101,7 +1008,7 @@ class Search:
 
         self.popup_menu_users.clear()
 
-        if len(self.selected_users) > 0:
+        if self.selected_users:
 
             items = []
 
@@ -1127,7 +1034,7 @@ class Search:
 
     def on_select_user_results(self, widget):
 
-        if len(self.selected_users) == 0:
+        if not self.selected_users:
             return
 
         selected_user = widget.get_parent().user
@@ -1208,10 +1115,10 @@ class Search:
 
     def on_key_press_event(self, widget, event):
 
-        key = Gdk.keyval_name(event.keyval)
         self.select_results()
 
-        if key in ("C", "c") and event.state in (Gdk.ModifierType.CONTROL_MASK, Gdk.ModifierType.LOCK_MASK | Gdk.ModifierType.CONTROL_MASK):
+        if event.get_state() & Gdk.ModifierType.CONTROL_MASK and \
+                event.hardware_keycode in keyval_to_hardware_keycode(Gdk.KEY_c):
             self.on_copy_file_path(widget)
         else:
             # No key match, continue event
@@ -1355,7 +1262,7 @@ class Search:
 
         directories = choose_dir(self.frame.MainWindow, self.frame.np.config.sections["transfers"]["downloaddir"], multichoice=False)
 
-        if directories is None or directories == []:
+        if not directories:
             return
 
         destination = directories[0]
@@ -1401,16 +1308,12 @@ class Search:
 
         self.on_refilter(widget)
 
-        self.ResultsList.set_show_expanders(widget.get_active())
+        active = widget.get_active()
 
-        self.frame.np.config.sections["searches"]["group_searches"] = widget.get_active()
-
-        if widget.get_active():
-            self.cols["id"].set_visible(False)
-            self.ExpandButton.show()
-        else:
-            self.cols["id"].set_visible(True)
-            self.ExpandButton.hide()
+        self.ResultsList.set_show_expanders(active)
+        self.frame.np.config.sections["searches"]["group_searches"] = active
+        self.cols["id"].set_visible(not active)
+        self.ExpandButton.set_visible(active)
 
     def on_toggle_expand_all(self, widget):
 
@@ -1431,19 +1334,6 @@ class Search:
         self.FiltersContainer.set_visible(visible)
         self.frame.np.config.sections["searches"]["filters_visible"] = visible
 
-    def on_clear(self, widget):
-        self.all_data = []
-        self.usersiters.clear()
-        self.directoryiters.clear()
-        self.resultsmodel.clear()
-        self.numvisibleresults = 0
-
-        # Update number of visible results
-        self.update_result_counter()
-
-    def on_close(self, widget):
-        self.searches.remove_tab(self)
-
     def on_copy_search_term(self, widget):
         self.frame.clip.set_text(self.text, -1)
 
@@ -1542,3 +1432,19 @@ class Search:
             self.FilterLabel.set_text(_("Result Filters"))
 
         self.FilterLabel.set_tooltip_text("%d active filter(s)" % count)
+
+    def on_clear(self, widget):
+        self.all_data = []
+        self.usersiters.clear()
+        self.directoryiters.clear()
+        self.resultsmodel.clear()
+        self.numvisibleresults = 0
+
+        # Update number of visible results
+        self.update_result_counter()
+
+    def on_close(self, widget):
+        self.searches.remove_tab(self)
+
+    def on_close_all_tabs(self, widget):
+        self.searches.remove_all_pages()
