@@ -23,7 +23,6 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import os
-import _thread
 
 from sys import maxsize
 from time import time
@@ -36,6 +35,7 @@ from pynicotine.gtkgui.utils import collapse_treeview
 from pynicotine.gtkgui.utils import human_size
 from pynicotine.gtkgui.utils import human_speed
 from pynicotine.gtkgui.utils import initialise_columns
+from pynicotine.gtkgui.utils import keyval_to_hardware_keycode
 from pynicotine.gtkgui.utils import load_ui_elements
 from pynicotine.gtkgui.utils import PopupMenu
 from pynicotine.gtkgui.utils import save_columns
@@ -57,32 +57,26 @@ class TransferList:
         self.frame.__dict__[type + "svbox"].add(self.Main)
         self.widget = widget = self.__dict__[type.title() + "List"]
 
-        self.last_ui_update = self.last_save = time()
+        self.last_ui_update = self.last_save = 0
         self.list = []
         self.users = {}
         self.paths = {}
 
         # Status list
         self.statuses = {}
-        self.statuses["Waiting for download"] = _("Waiting for download")
-        self.statuses["Requesting file"] = _("Requesting file")
-        self.statuses["Initializing transfer"] = _("Initializing transfer")
-        self.statuses["Cannot connect"] = _("Cannot connect")
-        self.statuses["Waiting for peer to connect"] = _("Waiting for peer to connect")
-        self.statuses["Connecting"] = _("Connecting")
-        self.statuses["Getting address"] = _("Getting address")
-        self.statuses["Getting status"] = _("Getting status")
         self.statuses["Queued"] = _("Queued")
+        self.statuses["Getting status"] = _("Getting status")
+        self.statuses["Establishing connection"] = _("Establishing connection")
+        self.statuses["Transferring"] = _("Transferring")
+        self.statuses["Cannot connect"] = _("Cannot connect")
         self.statuses["User logged off"] = _("User logged off")
+        self.statuses["Connection closed by peer"] = _("Connection closed by peer")
         self.statuses["Aborted"] = _("Aborted")
         self.statuses["Finished"] = _("Finished")
         self.statuses["Paused"] = _("Paused")
-        self.statuses["Transferring"] = _("Transferring")
         self.statuses["Filtered"] = _("Filtered")
-        self.statuses["Connection closed by peer"] = _("Connection closed by peer")
         self.statuses["File not shared"] = _("File not shared")
         self.statuses["File not shared."] = _("File not shared")  # The official client sends a variant containing a dot
-        self.statuses["Establishing connection"] = _("Establishing connection")
         self.statuses["Download directory error"] = _("Download directory error")
         self.statuses["Local file error"] = _("Local file error")
         self.statuses["Remote file error"] = _("Remote file error")
@@ -112,7 +106,7 @@ class TransferList:
             GObject.TYPE_UINT64,   # (17) queue position
         )
 
-        self.column_numbers = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17]
+        self.column_numbers = list(range(self.transfersmodel.get_n_columns()))
         self.cols = cols = initialise_columns(
             type,
             widget,
@@ -162,8 +156,8 @@ class TransferList:
 
     def init_interface(self, list):
         self.list = list
-        self.update()
         self.widget.set_sensitive(True)
+        self.update()
 
     def conn_close(self):
         self.widget.set_sensitive(False)
@@ -183,7 +177,7 @@ class TransferList:
         self.select_transfers()
 
         for user in self.selected_users:
-            self.frame.ban_user(user)
+            self.frame.np.network_filter.ban_user(user)
 
     def on_file_search(self, widget):
 
@@ -238,6 +232,10 @@ class TransferList:
 
     def update(self, transfer=None, forceupdate=False):
 
+        if not self.widget.get_sensitive():
+            """ List is not initialized """
+            return
+
         curtime = time()
 
         if (curtime - self.last_save) > 15:
@@ -261,8 +259,8 @@ class TransferList:
 
         if transfer is not None:
             self.update_specific(transfer)
-        elif self.list is not None:
 
+        elif self.list is not None:
             for i in self.list:
                 self.update_specific(i)
 
@@ -364,7 +362,7 @@ class TransferList:
         if elapsed > 0:
             helapsed = self.frame.np.transfers.get_time(elapsed)
 
-        if len(extensions) == 0:
+        if not extensions:
             extensions = ""
         elif len(extensions) == 1:
             extensions = " (" + self.extension_list_template % {'ext': next(iter(extensions))} + ")"
@@ -581,6 +579,10 @@ class TransferList:
 
                 self.expand(transfer_path, user_path)
 
+    def retry_transfers(self):
+        for transfer in self.selected_transfers:
+            getattr(self.frame.np.transfers, "retry_" + self.type)(transfer)
+
     def abort_transfers(self, clear=False):
 
         for i in self.selected_transfers:
@@ -596,6 +598,9 @@ class TransferList:
 
     def remove_specific(self, transfer, cleartreeviewonly=False):
 
+        if transfer in self.frame.np.transfers.transfer_request_times:
+            del self.frame.np.transfers.transfer_request_times[transfer]
+
         if not cleartreeviewonly:
             self.list.remove(transfer)
 
@@ -608,9 +613,6 @@ class TransferList:
 
         for i in self.list[:]:
             if i.status in status:
-                if i.transfertimer is not None:
-                    i.transfertimer.cancel()
-
                 if i.status == "Queued":
                     self.frame.np.transfers.abort_transfer(i)
 
@@ -641,7 +643,7 @@ class TransferList:
     def on_expand_tree(self, widget):
 
         expand_button_icon = self.frame.__dict__["Expand%ssImage" % self.type.title()]
-        expanded = self.expand_button.get_active()
+        expanded = widget.get_active()
 
         if expanded:
             self.widget.expand_all()
@@ -655,16 +657,13 @@ class TransferList:
 
     def on_toggle_tree(self, widget):
 
-        pos = self.group_dropdown.get_active()
-        self.frame.np.config.sections["transfers"]["group%ss" % self.type] = pos
+        active = widget.get_active()
 
-        self.tree_users = self.group_dropdown.get_active_id()
-
-        if self.tree_users == "ungrouped":
-            self.expand_button.hide()
-        else:
-            self.expand_button.show()
+        self.frame.np.config.sections["transfers"]["group%ss" % self.type] = active
+        self.widget.set_show_expanders(active)
+        self.expand_button.set_visible(active)
 
+        self.tree_users = widget.get_active_id()
         self.rebuild_transfers()
 
     def on_tooltip(self, widget, x, y, keyboard_mode, tooltip):
@@ -686,7 +685,7 @@ class TransferList:
 
         self.popup_menu_users.clear()
 
-        if len(self.selected_users) > 0:
+        if self.selected_users:
 
             items = []
 
@@ -713,7 +712,7 @@ class TransferList:
 
     def on_select_user_transfers(self, widget):
 
-        if len(self.selected_users) == 0:
+        if not self.selected_users:
             return
 
         selected_user = widget.get_parent().user
@@ -728,8 +727,30 @@ class TransferList:
 
         self.select_transfers()
 
-    def on_play_files(self, widget, prefix=""):
-        _thread.start_new_thread(self._on_play_files, (widget, prefix))
+    def on_key_press_event(self, widget, event):
+
+        keycode = event.hardware_keycode
+        self.select_transfers()
+
+        if keycode in keyval_to_hardware_keycode(Gdk.KEY_t):
+            self.on_abort_transfer(widget)
+
+        elif keycode in keyval_to_hardware_keycode(Gdk.KEY_r):
+            self.on_retry_transfer(widget)
+
+        elif event.get_state() & Gdk.ModifierType.CONTROL_MASK and \
+                keycode in keyval_to_hardware_keycode(Gdk.KEY_c):
+            self.on_copy_file_path(widget)
+
+        elif keycode in keyval_to_hardware_keycode(Gdk.KEY_Delete):
+            self.on_abort_transfer(widget, clear=True)
+
+        else:
+            # No key match, continue event
+            return False
+
+        widget.stop_emission_by_name("key_press_event")
+        return True
 
     def on_copy_file_path(self, widget):
 
@@ -755,6 +776,14 @@ class TransferList:
 
         self.frame.set_clipboard_url(i.user, path)
 
+    def on_retry_transfer(self, widget):
+        self.select_transfers()
+        self.retry_transfers()
+
+    def on_abort_transfer(self, widget, clear=False):
+        self.select_transfers()
+        self.abort_transfers(clear)
+
     def on_clear_transfer(self, widget):
         self.select_transfers()
         self.abort_transfers(clear=True)
@@ -765,6 +794,9 @@ class TransferList:
 
         dialog.destroy()
 
+    def on_clear_queued(self, widget):
+        self.clear_transfers(["Queued"])
+
     def on_clear_finished(self, widget):
         self.clear_transfers(["Finished"])
 
