@@ -24,8 +24,6 @@
 
 import os
 
-from gi.repository import Gdk
-
 from pynicotine.gtkgui.dialogs import option_dialog
 from pynicotine.gtkgui.transferlist import TransferList
 from pynicotine.gtkgui.utils import open_file_path
@@ -64,7 +62,7 @@ class Uploads(TransferList):
             ("#" + _("_Search"), self.on_file_search),
             (1, _("User(s)"), self.popup_menu_users, self.on_popup_menu_users),
             ("", None),
-            ("#" + _("_Retry"), self.on_upload_transfer),
+            ("#" + _("_Retry"), self.on_retry_transfer),
             ("#" + _("Abor_t"), self.on_abort_transfer),
             ("#" + _("_Clear"), self.on_clear_transfer),
             ("", None),
@@ -98,26 +96,7 @@ class Uploads(TransferList):
         command = self.frame.np.config.sections["ui"]["filemanager"]
         open_file_path(final_path, command)
 
-    def on_key_press_event(self, widget, event):
-
-        key = Gdk.keyval_name(event.keyval)
-
-        self.select_transfers()
-
-        if key in ("T", "t"):
-            self.on_abort_transfer(widget)
-        elif key in ("C", "c") and event.state in (Gdk.ModifierType.CONTROL_MASK, Gdk.ModifierType.LOCK_MASK | Gdk.ModifierType.CONTROL_MASK):
-            self.on_copy_file_path(widget)
-        elif key == "Delete":
-            self.on_abort_transfer(widget, clear=True)
-        else:
-            # No key match, continue event
-            return False
-
-        widget.stop_emission_by_name("key_press_event")
-        return True
-
-    def _on_play_files(self, widget, prefix=""):
+    def on_play_files(self, widget, prefix=""):
 
         for fn in self.selected_transfers:
             playfile = fn.realfilename
@@ -184,22 +163,8 @@ class Uploads(TransferList):
 
         for i in self.list[:]:
             if i.user == user:
-                if i.transfertimer is not None:
-                    i.transfertimer.cancel()
                 self.remove_specific(i)
 
-        self.frame.np.transfers.calc_upload_queue_sizes()
-        self.frame.np.transfers.check_upload_queue()
-
-    def on_abort_transfer(self, widget, clear=False):
-
-        self.select_transfers()
-
-        self.abort_transfers(clear)
-
-        self.frame.np.transfers.calc_upload_queue_sizes()
-        self.frame.np.transfers.check_upload_queue()
-
     def on_abort_user(self, widget):
 
         self.select_transfers()
@@ -210,28 +175,6 @@ class Uploads(TransferList):
                     self.selected_transfers.add(i)
 
         self.on_abort_transfer(widget)
-        self.frame.np.transfers.calc_upload_queue_sizes()
-        self.frame.np.transfers.check_upload_queue()
-
-    def on_clear_queued(self, widget):
-
-        self.clear_transfers(["Queued"])
-
-        self.frame.np.transfers.calc_upload_queue_sizes()
-        self.frame.np.transfers.check_upload_queue()
 
     def on_clear_failed(self, widget):
-
-        self.clear_transfers(["Cannot connect", "Connection closed by peer", "Local file error", "Remote file error", "Getting address", "Waiting for peer to connect", "Initializing transfer"])
-
-        self.frame.np.transfers.calc_upload_queue_sizes()
-        self.frame.np.transfers.check_upload_queue()
-
-    def on_upload_transfer(self, widget):
-
-        self.select_transfers()
-
-        for transfer in self.selected_transfers:
-            self.frame.np.transfers.retry_upload(transfer)
-
-        self.frame.np.transfers.check_upload_queue()
+        self.clear_transfers(["Cannot connect", "Connection closed by peer", "Local file error", "Remote file error"])
