@@ -29,7 +29,6 @@ from gi.repository import Gdk
 from gi.repository import GObject
 from gi.repository import Gtk
 
-from _thread import start_new_thread
 from pynicotine import slskmessages
 from pynicotine.gtkgui.dialogs import choose_dir
 from pynicotine.gtkgui.dialogs import combo_box_dialog
@@ -37,6 +36,7 @@ from pynicotine.gtkgui.fileproperties import FileProperties
 from pynicotine.gtkgui.utils import human_size
 from pynicotine.gtkgui.utils import InfoBar
 from pynicotine.gtkgui.utils import initialise_columns
+from pynicotine.gtkgui.utils import keyval_to_hardware_keycode
 from pynicotine.gtkgui.utils import load_ui_elements
 from pynicotine.gtkgui.utils import open_file_path
 from pynicotine.gtkgui.utils import PopupMenu
@@ -45,7 +45,7 @@ from pynicotine.gtkgui.utils import set_treeview_selected_row
 from pynicotine.gtkgui.utils import triggers_context_menu
 from pynicotine.gtkgui.utils import update_widget_visuals
 from pynicotine.logfacility import log
-from pynicotine.utils import clean_file
+from pynicotine.utils import get_path
 from pynicotine.utils import get_result_bitrate_length
 
 
@@ -61,8 +61,7 @@ class UserBrowse:
         self.info_bar = InfoBar(self.InfoBar, Gtk.MessageType.INFO)
 
         # Monitor user online status
-        if user not in self.frame.np.watchedusers:
-            self.frame.np.queue.put(slskmessages.AddUser(user))
+        self.frame.np.watch_user(user)
 
         self.user = user
         self.conn = None
@@ -91,7 +90,7 @@ class UserBrowse:
 
         self.dir_store = Gtk.TreeStore(str, str)
 
-        self.dir_column_numbers = [0, 1]
+        self.dir_column_numbers = list(range(self.dir_store.get_n_columns()))
         cols = initialise_columns(
             None,
             self.FolderTreeView,
@@ -110,7 +109,7 @@ class UserBrowse:
 
             menu.append_item(("", None))
             menu.append_item(("#" + _("_Save Shares List To Disk"), self.on_save))
-            menu.append_item(("#" + _("Close All Tabs"), menu.on_close_all_tabs, self.userbrowses))
+            menu.append_item(("#" + _("Close All Tabs"), self.on_close_all_tabs))
             menu.append_item(("#" + _("_Close Tab"), self.on_close))
 
         self.popup_menu_downloads_folders = PopupMenu(self.frame, False)
@@ -191,7 +190,7 @@ class UserBrowse:
 
         self.FileTreeView.set_model(self.file_store)
 
-        self.file_column_numbers = [0, 1, 2, 3, 4, 5, 6]
+        self.file_column_numbers = [i for i in range(self.file_store.get_n_columns())]
         cols = initialise_columns(
             "user_browse",
             self.FileTreeView,
@@ -270,7 +269,7 @@ class UserBrowse:
 
             dirs = sorted(self.directories.keys())
 
-            if dirs != []:
+            if dirs:
                 self.set_directory(dirs[0])
             else:
                 self.set_directory(None)
@@ -289,6 +288,19 @@ class UserBrowse:
         return False
 
     def on_folder_popup_menu(self, widget):
+
+        items = self.folder_popup_menu.get_items()
+
+        if self.user == self.frame.np.config.sections["server"]["login"]:
+            for i in (_("_Download Folder"), _("Download Folder _To..."), _("Download _Recursive"), _("Download R_ecursive To..."),
+                      _("Upload Folder To..."), _("Upload Folder Recursive To..."), _("Open in File _Manager"),
+                      _("Copy _Folder Path"), _("Copy _URL")):
+                items[i].set_sensitive(self.selected_folder)
+        else:
+            for i in (_("_Download Folder"), _("Download Folder _To..."), _("Download _Recursive"), _("Download R_ecursive To..."),
+                      _("Copy _Folder Path"), _("Copy _URL")):
+                items[i].set_sensitive(self.selected_folder)
+
         self.folder_popup_menu.popup()
         return True
 
@@ -322,20 +334,17 @@ class UserBrowse:
         self.select_files()
         num_selected_files = len(self.selected_files)
 
-        if num_selected_files >= 1:
-            files = True
-        else:
-            files = False
-
         items = self.file_popup_menu.get_items()
 
         if self.user == self.frame.np.config.sections["server"]["login"]:
             for i in (_("Download"), _("Upload"), _("Send to _Player"), _("File _Properties"),
                       _("Copy _File Path"), _("Copy _URL")):
-                items[i].set_sensitive(files)
+                items[i].set_sensitive(num_selected_files)
+
+            items[_("Open in File _Manager")].set_sensitive(self.selected_folder)
         else:
             for i in (_("Download"), _("File _Properties"), _("Copy _File Path"), _("Copy _URL")):
-                items[i].set_sensitive(files)
+                items[i].set_sensitive(num_selected_files)
 
         items["selected_files"].set_sensitive(False)
         items["selected_files"].set_label(_("%s File(s) Selected") % num_selected_files)
@@ -351,6 +360,7 @@ class UserBrowse:
         self.directories.clear()
         self.files.clear()
         self.dir_store.clear()
+        self.file_store.clear()
 
         # Compute the number of shared dirs and total size
         self.totalsize = 0
@@ -367,6 +377,7 @@ class UserBrowse:
 
         sel = self.FolderTreeView.get_selection()
         sel.unselect_all()
+
         if currentdir in self.directories:
             path = self.dir_store.get_path(self.directories[currentdir])
             if path is not None:
@@ -385,8 +396,7 @@ class UserBrowse:
         dirseparator = '\\'
 
         # If there is no share
-
-        if self.shares == []:
+        if not self.shares:
 
             # Set the model of the treeviex
             self.FolderTreeView.set_model(self.dir_store)
@@ -437,7 +447,7 @@ class UserBrowse:
                 )
 
                 # If there are subdirs futher down the path: recurse
-                if len(dictdir[subdir]):
+                if dictdir[subdir]:
                     buildgtktree(dictdir[subdir], self.directories[current_path], current_path)
 
         # For each shared dir we will complete the dictionnary
@@ -475,7 +485,7 @@ class UserBrowse:
             iterator = self.directories[folder]
         except KeyError:
             # Folder not found
-            pass
+            return
 
         if folder:
             sel = self.FolderTreeView.get_selection()
@@ -543,17 +553,18 @@ class UserBrowse:
             log.add(_("Can't create directory '%(folder)s', reported error: %(error)s"), {'folder': sharesdir, 'error': msg})
 
         try:
-            filepath = os.path.join(sharesdir, clean_file(self.user))
-
-            with open(filepath, "w", encoding="utf-8") as sharesfile:
-                import json
-                json.dump(self.shares, sharesfile, ensure_ascii=False)
-
+            get_path(sharesdir, self.user, self.dump_shares_to_file)
             log.add(_("Saved list of shared files for user '%(user)s' to %(dir)s"), {'user': self.user, 'dir': sharesdir})
 
         except Exception as msg:
             log.add(_("Can't save shares, '%(user)s', reported error: %(error)s"), {'user': self.user, 'error': msg})
 
+    def dump_shares_to_file(self, path, data):
+
+        with open(path, "w", encoding="utf-8") as sharesfile:
+            import json
+            json.dump(self.shares, sharesfile, ensure_ascii=False)
+
     def save_columns(self):
         save_columns("user_browse", self.FileTreeView.get_columns())
 
@@ -580,7 +591,7 @@ class UserBrowse:
 
             self.make_new_model(msg.list)
 
-        if msg and len(msg.list) == 0:
+        if msg and not msg.list:
             self.info_bar.show_message(
                 _("User's list of shared files is empty. Either the user is not sharing anything, or they are sharing files privately.")
             )
@@ -648,7 +659,6 @@ class UserBrowse:
         model, iterator = selection.get_selected()
 
         if iterator is None:
-            self.selected_folder = None
             return
 
         path = model.get_path(iterator)
@@ -727,7 +737,7 @@ class UserBrowse:
         ldir = prefix + folder.split("\\")[-1]
 
         # Check if folder already exists on system
-        ldir = self.frame.np.transfers.folder_destination(self.user, ldir)
+        ldir = self.frame.np.transfers.get_folder_destination(self.user, ldir)
 
         for d, files in self.shares:
 
@@ -852,6 +862,7 @@ class UserBrowse:
         realpath = self.frame.np.shares.virtual2real(folder)
         ldir = folder.split("\\")[-1]
 
+        locally_queued = False
         for d, f in self.shares:
 
             # Find the wanted directory
@@ -860,10 +871,10 @@ class UserBrowse:
 
             for file in f:
                 filename = "\\".join([folder, file[1]])
-                realfilename = "\\".join([realpath, file[1]])
+                realfilename = os.path.join(realpath, file[1])
                 size = file[2]
-                self.frame.np.transfers.push_file(user, filename, realfilename, ldir, size=size)
-                self.frame.np.transfers.check_upload_queue()
+                self.frame.np.transfers.push_file(user, filename, realfilename, ldir, size=size, locally_queued=locally_queued)
+                locally_queued = True
 
         if not recurse:
             return
@@ -898,18 +909,20 @@ class UserBrowse:
 
         self.frame.np.send_message_to_peer(user, slskmessages.UploadQueueNotification(None))
 
+        locally_queued = False
         for fn in self.selected_files:
-            self.frame.np.transfers.push_file(user, "\\".join([folder, fn]), "\\".join([realpath, fn]), prefix)
-            self.frame.np.transfers.check_upload_queue()
+            self.frame.np.transfers.push_file(user, "\\".join([folder, fn]), os.path.join(realpath, fn), prefix, locally_queued=locally_queued)
+            locally_queued = True
 
     def on_key_press_event(self, widget, event):
 
-        key = Gdk.keyval_name(event.keyval)
         self.select_files()
 
-        if key in ("C", "c") and event.state in (Gdk.ModifierType.CONTROL_MASK, Gdk.ModifierType.LOCK_MASK | Gdk.ModifierType.CONTROL_MASK):
+        if event.get_state() & Gdk.ModifierType.CONTROL_MASK and \
+                event.hardware_keycode in keyval_to_hardware_keycode(Gdk.KEY_c):
             files = (widget == self.FileTreeView)
             self.on_copy_file_path(widget, files)
+
         else:
             # No key match, continue event
             return False
@@ -918,9 +931,6 @@ class UserBrowse:
         return True
 
     def on_play_files(self, widget, prefix=""):
-        start_new_thread(self._on_play_files, (widget, prefix))
-
-    def _on_play_files(self, widget, prefix=""):
 
         path = self.frame.np.shares.virtual2real(self.selected_folder)
 
@@ -959,7 +969,7 @@ class UserBrowse:
                 return
             self.find_matches()
 
-        if self.search_list != []:
+        if self.search_list:
 
             if self.search_position not in list(range(len(self.search_list))):
                 self.search_position = 0
@@ -995,10 +1005,6 @@ class UserBrowse:
         else:
             self.search_position = 0
 
-    def on_close(self, widget):
-        del self.userbrowses.users[self.user]
-        self.userbrowses.remove_page(self.Main)
-
     def on_refresh(self, widget):
 
         self.refreshing = True
@@ -1011,6 +1017,9 @@ class UserBrowse:
 
     def on_copy_file_path(self, widget, files=False):
 
+        if self.selected_folder is None:
+            return
+
         text = self.selected_folder
 
         if files and self.selected_files:
@@ -1020,7 +1029,7 @@ class UserBrowse:
 
     def on_copy_url(self, widget):
 
-        if self.selected_files != [] and self.selected_files is not None:
+        if self.selected_files:
             path = "\\".join([self.selected_folder, self.selected_files[0]])
             self.frame.set_clipboard_url(self.user, path)
 
@@ -1044,3 +1053,10 @@ class UserBrowse:
         command = self.frame.np.config.sections["ui"]["filemanager"]
 
         open_file_path(path, command)
+
+    def on_close(self, widget):
+        del self.userbrowses.users[self.user]
+        self.userbrowses.remove_page(self.Main)
+
+    def on_close_all_tabs(self, widget):
+        self.userbrowses.remove_all_pages()
