@@ -87,6 +87,11 @@ class UserTabs(IconNotebook):
 
         if user in self.users:
             self.users[user].conn = conn
+
+        elif not change_page:
+            # This tab was closed, but we received a response. Don't reopen it.
+            return
+
         else:
             self.init_window(user)
 
@@ -160,30 +165,14 @@ class UserTabs(IconNotebook):
         menu.popup()
         return True
 
-    def on_tab_click(self, widget, event, page):
+    def login(self):
 
-        if triggers_context_menu(event):
-            return self.on_tab_popup(widget, page)
-
-        if event.button == 2:
-            username = self.get_page_owner(page, self.users)
-            self.users[username].on_close(widget)
-            return True
-
-        return False
-
-    def close_all_tabs(self, dialog, response, data):
-
-        if response == Gtk.ResponseType.OK:
-            for user in self.users.copy():
-                self.users[user].on_close(dialog)
-
-        dialog.destroy()
+        for user in self.users:
+            # Get notified of user status
+            self.frame.np.watch_user(user)
 
     def conn_close(self):
 
-        self.connected = 0
-
         for user in self.users:
             tab = self.users[user]
             tab.status = 0
@@ -203,10 +192,10 @@ class UserInfo:
         self.info_bar = InfoBar(self.InfoBar, Gtk.MessageType.INFO)
 
         # Request user status, speed and number of shared files
-        self.frame.np.queue.put(slskmessages.AddUser(user))
+        self.frame.np.watch_user(user, force_update=True)
 
         # Request user interests
-        self.frame.np.queue.put(slskmessages.UserInterests(user))
+        self.frame.np.queue.append(slskmessages.UserInterests(user))
 
         self.user = user
         self.conn = None
@@ -249,7 +238,7 @@ class UserInfo:
         popup.get_items()[_("Show User I_nfo")].set_visible(False)
 
         popup.append_item(("", None))
-        popup.append_item(("#" + _("Close All Tabs"), popup.on_close_all_tabs, self.userinfos))
+        popup.append_item(("#" + _("Close All Tabs"), self.on_close_all_tabs))
         popup.append_item(("#" + _("_Close Tab"), self.on_close))
 
         self.likes_popup_menu = popup = PopupMenu(self.frame)
@@ -371,7 +360,7 @@ class UserInfo:
             self.SavePicture.set_sensitive(True)
 
         except Exception as e:
-            log.add(_("Failed to load picture for user %(user)s: %(error)s") % {
+            log.add(_("Failed to load picture for user %(user)s: %(error)s"), {
                 "user": self.user,
                 "error": str(e)
             })
@@ -387,22 +376,22 @@ class UserInfo:
 
         append_line(self.descr, msg.descr, self.tag_local, showstamp=False, scroll=False)
 
-        self.uploads.set_text(_("Total uploads allowed: %i") % msg.totalupl)
-        self.queuesize.set_text(_("Queue size: %i") % msg.queuesize)
+        self.uploads.set_text(_("Upload slots: %i") % msg.totalupl)
+        self.queuesize.set_text(_("Queued uploads: %i") % msg.queuesize)
 
         if msg.slotsavail:
             slots = _("Yes")
         else:
             slots = _("No")
 
-        self.slotsavail.set_text(_("Slots free: %s") % slots)
+        self.slotsavail.set_text(_("Free upload slots: %s") % slots)
 
         if msg.uploadallowed == 0:
             allowed = _("No one")
         elif msg.uploadallowed == 1:
             allowed = _("Everyone")
         elif msg.uploadallowed == 2:
-            allowed = _("Users in list")
+            allowed = _("Buddies")
         elif msg.uploadallowed == 3:
             allowed = _("Trusted Users")
         else:
@@ -456,7 +445,7 @@ class UserInfo:
     def on_show_ip_address(self, widget):
 
         self.frame.np.ip_requested.add(self.user)
-        self.frame.np.queue.put(slskmessages.GetPeerAddress(self.user))
+        self.frame.np.queue.append(slskmessages.GetPeerAddress(self.user))
 
     def on_refresh(self, widget):
         self.info_bar.set_visible(False)
@@ -470,14 +459,10 @@ class UserInfo:
         self.frame.np.userlist.add_to_list(self.user)
 
     def on_ban_user(self, widget):
-        self.frame.ban_user(self.user)
+        self.frame.np.network_filter.ban_user(self.user)
 
     def on_ignore_user(self, widget):
-        self.frame.ignore_user(self.user)
-
-    def on_close(self, widget):
-        del self.userinfos.users[self.user]
-        self.userinfos.remove_page(self.Main)
+        self.frame.np.network_filter.ignore_user(self.user)
 
     def on_save_picture(self, widget):
 
@@ -585,3 +570,10 @@ class UserInfo:
         del pixbuf_zoomed
 
         gc.collect()
+
+    def on_close(self, widget):
+        del self.userinfos.users[self.user]
+        self.userinfos.remove_page(self.Main)
+
+    def on_close_all_tabs(self, widget):
+        self.userinfos.remove_all_pages()
