@@ -39,6 +39,7 @@ from pynicotine.gtkgui.utils import PopupMenu
 from pynicotine.gtkgui.utils import save_columns
 from pynicotine.gtkgui.utils import set_treeview_selected_row
 from pynicotine.gtkgui.utils import show_country_tooltip
+from pynicotine.gtkgui.utils import show_user_status_tooltip
 from pynicotine.gtkgui.utils import triggers_context_menu
 from pynicotine.gtkgui.utils import update_widget_visuals
 from pynicotine.logfacility import log
@@ -67,14 +68,14 @@ class UserList:
             bool,                 # (7)  privileged
             str,                  # (8)  hlast seen
             str,                  # (9)  comments
-            GObject.TYPE_INT64,   # (10) status
+            int,                  # (10) status
             GObject.TYPE_UINT64,  # (11) speed
             GObject.TYPE_UINT64,  # (12) file count
             int,                  # (13) last seen
             str                   # (14) country
         )
 
-        self.column_numbers = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14]
+        self.column_numbers = list(range(self.usersmodel.get_n_columns()))
         self.cols = cols = initialise_columns(
             "buddy_list",
             self.UserListTree,
@@ -193,7 +194,15 @@ class UserList:
                 widget.append_text(str(user[0]))
 
     def on_tooltip(self, widget, x, y, keyboard_mode, tooltip):
-        return show_country_tooltip(widget, x, y, tooltip, 14)
+
+        status_tooltip = show_user_status_tooltip(widget, x, y, tooltip, 10)
+        country_tooltip = show_country_tooltip(widget, x, y, tooltip, 14)
+
+        if status_tooltip:
+            return status_tooltip
+
+        if country_tooltip:
+            return country_tooltip
 
     def on_add_user(self, widget, *args):
 
@@ -446,10 +455,12 @@ class UserList:
         )
 
         self.save_user_list()
-        self.frame.np.queue.put(slskmessages.AddUser(user))
+
+        # Request user status, speed and number of shared files
+        self.frame.np.watch_user(user, force_update=True)
 
         # Request user's IP address, so we can get the country
-        self.frame.np.queue.put(slskmessages.GetPeerAddress(user))
+        self.frame.np.queue.append(slskmessages.GetPeerAddress(user))
 
         for widget in self.buddies_combo_entries:
             widget.append_text(user)
