@@ -44,8 +44,8 @@ from pynicotine.gtkgui.dialogs import choose_image
 from pynicotine.gtkgui.dialogs import entry_dialog
 from pynicotine.gtkgui.dialogs import option_dialog
 from pynicotine.logfacility import log
-from pynicotine.utils import clean_file
 from pynicotine.utils import execute_command
+from pynicotine.utils import get_path
 
 
 URL_RE = re.compile("(\\w+\\://[^\\s]+)|(www\\.\\w+\\.\\w+.*?)|(mailto\\:[^\\s]+)")
@@ -396,7 +396,7 @@ def set_treeview_selected_row(treeview, event):
         selection.unselect_all()
 
 
-def show_country_tooltip(treeview, x, y, tooltip, sourcecolumn, stripprefix='flag_'):
+def show_tooltip(treeview, x, y, tooltip, sourcecolumn, column_titles, text_function, strip_prefix=""):
 
     try:
         bin_x, bin_y = treeview.convert_widget_to_bin_window_coords(x, y)
@@ -405,58 +405,61 @@ def show_country_tooltip(treeview, x, y, tooltip, sourcecolumn, stripprefix='fla
     except TypeError:
         return False
 
-    if column.get_title() != "country":
+    if column.get_title() not in column_titles:
         return False
 
     model = treeview.get_model()
     iterator = model.get_iter(path)
-    value = model.get_value(iterator, sourcecolumn)
-
-    # Avoid throwing an error in there's no flag
-    if value is None:
-        return False
+    column_value = model.get_value(iterator, sourcecolumn)
 
     # Update tooltip position
     treeview.set_tooltip_cell(tooltip, path, column, None)
 
-    if not value.startswith(stripprefix):
-        tooltip.set_text(_("Unknown"))
-        return True
+    text = text_function(column_value, strip_prefix)
+    if not text:
+        return False
 
-    value = value[len(stripprefix):]
-    if value:
-        countryname = code2name(value)
-    else:
-        countryname = _("Earth")
-
-    tooltip.set_text(countryname)
+    tooltip.set_text(text)
     return True
 
 
+def get_country_tooltip_text(column_value, strip_prefix):
+
+    if not column_value.startswith(strip_prefix):
+        return _("Unknown")
+
+    column_value = column_value[len(strip_prefix):]
+    if column_value:
+        return code2name(column_value)
+
+    return _("Earth")
+
+
+def get_file_path_tooltip_text(column_value, strip_prefix):
+    return column_value
+
+
+def get_user_status_tooltip_text(column_value, strip_prefix):
+
+    if column_value == 1:
+        return _("Away")
+
+    if column_value == 2:
+        return _("Online")
+
+    return _("Offline")
+
+
+def show_country_tooltip(treeview, x, y, tooltip, sourcecolumn, strip_prefix='flag_'):
+    return show_tooltip(treeview, x, y, tooltip, sourcecolumn, ("country",), get_country_tooltip_text, strip_prefix)
+
+
 def show_file_path_tooltip(treeview, x, y, tooltip, sourcecolumn):
+    return show_tooltip(treeview, x, y, tooltip, sourcecolumn, ("folder", "filename"), get_file_path_tooltip_text)
 
-    try:
-        bin_x, bin_y = treeview.convert_widget_to_bin_window_coords(x, y)
-        path, column, cx, cy = treeview.get_path_at_pos(bin_x, bin_y)
 
-    except TypeError:
-        return False
-
-    if column.get_title() not in ("folder", "filename"):
-        return False
-
-    model = treeview.get_model()
-    iterator = model.get_iter(path)
-    value = model.get_value(iterator, sourcecolumn)
-
-    if not value:
-        return False
-
-    # Update tooltip position
-    treeview.set_tooltip_cell(tooltip, path, column, None)
-
-    tooltip.set_text(value)
-    return True
+def show_user_status_tooltip(treeview, x, y, tooltip, sourcecolumn):
+    return show_tooltip(treeview, x, y, tooltip, sourcecolumn, ("status",), get_user_status_tooltip_text)
 
 
 def open_file_path(file_path, command=None):
@@ -484,40 +487,43 @@ def open_file_path(file_path, command=None):
 
 
 def open_log(folder, filename):
+    _handle_log(folder, filename, open_log_callback)
+
+
+def delete_log(folder, filename):
+    _handle_log(folder, filename, delete_log_callback)
+
+
+def _handle_log(folder, filename, callback):
 
     try:
         if not os.path.isdir(folder):
             os.makedirs(folder)
 
-        path = os.path.join(folder, clean_file(filename.replace(os.sep, "-")) + ".log")
-
-        if not os.path.exists(path):
-            with open(path, "w"):
-                # No logs, create empty file
-                pass
-
-        open_file_path(path)
+        filename = filename.replace(os.sep, "-") + ".log"
+        get_path(folder, filename, callback)
 
     except Exception as e:
-        log.add("Failed to open log file: %s", e)
+        log.add("Failed to process log file: %s", e)
 
 
-def scroll_bottom(widget):
-    va = widget.get_vadjustment()
-    try:
-        va.set_value(va.get_upper() - va.get_page_size())
-    except AttributeError:
+def open_log_callback(path, data):
+
+    if not os.path.exists(path):
+        with open(path, "w"):
+            # No logs, create empty file
+            pass
+
+    open_file_path(path)
+
+
+def delete_log_callback(path, data):
+
+    with open(path, "w"):
+        # Check if path should contain special characters
         pass
-    widget.set_vadjustment(va)
-    return False
 
-
-def url_event(tag, widget, event, iterator, url):
-    if tag.last_event_type == Gdk.EventType.BUTTON_PRESS and event.button.type == Gdk.EventType.BUTTON_RELEASE and event.button.button == 1:
-        if url[:4] == "www.":
-            url = "http://" + url
-        open_uri(url, widget.get_toplevel())
-    tag.last_event_type = event.button.type
+    os.remove(path)
 
 
 def open_uri(uri, window):
@@ -569,112 +575,105 @@ def on_soul_seek_uri(url):
         log.add(_("Invalid SoulSeek meta-url: %s"), url)
 
 
+def scroll_bottom(widget):
+
+    if not NICOTINE.shutdown:
+        adjustment = widget.get_vadjustment()
+        adjustment.set_value(adjustment.get_upper() - adjustment.get_page_size())
+
+
+def url_event(tag, widget, event, iterator, url):
+
+    if tag.last_event_type == Gdk.EventType.BUTTON_PRESS and event.button.type == Gdk.EventType.BUTTON_RELEASE and event.button.button == 1:
+        if url[:4] == "www.":
+            url = "http://" + url
+        open_uri(url, widget.get_toplevel())
+
+    tag.last_event_type = event.button.type
+
+
 def append_line(textview, line, tag=None, timestamp=None, showstamp=True, timestamp_format="%H:%M:%S", username=None, usertag=None, scroll=True, find_urls=True):
 
-    if type(line) not in (type(""), type("")):
-        line = str(line)  # Error messages are sometimes tuples
-
     def _makeurltag(buffer, url):
-        props = {}
 
-        color = NICOTINE.np.config.sections["ui"]["urlcolor"]
-
-        if color != "":
-            props["foreground"] = color
-
-        props["underline"] = Pango.Underline.SINGLE
-        tag = buffer.create_tag(**props)
+        color = NICOTINE.np.config.sections["ui"]["urlcolor"] or None
+        tag = buffer.create_tag(foreground=color, underline=Pango.Underline.SINGLE)
         tag.last_event_type = -1
         tag.connect("event", url_event, url)
+
         return tag
 
     def _append(buffer, text, tag):
+
         iterator = buffer.get_end_iter()
+        start_offset = iterator.get_offset()
+        buffer.insert(iterator, text)
 
         if tag is not None:
-            buffer.insert_with_tags(iterator, text, tag)
-        else:
-            buffer.insert(iterator, text)
+            start = buffer.get_iter_at_offset(start_offset)
+            buffer.apply_tag(tag, start, iterator)
 
     def _usertag(buffer, section):
+
         # Tag usernames with popup menu creating tag, and away/online/offline colors
-        if NICOTINE.np.config.sections["ui"]["usernamehotspots"] and username is not None and usertag is not None:
+        if username is not None and usertag is not None and NICOTINE.np.config.sections["ui"]["usernamehotspots"]:
             np = re.compile(re.escape(str(username)))
             match = np.search(section)
+
             if match is not None:
-                start2 = section[:match.start()]
-                name = match.group()[:]
-                start = section[match.end():]
-                _append(buffer, start2, tag)
-                _append(buffer, name, usertag)
-                _append(buffer, start, tag)
-            else:
-                _append(buffer, section, tag)
-        else:
-            _append(buffer, section, tag)
+                _append(buffer, section[:match.start()], tag)
+                _append(buffer, match.group(), usertag)
+                _append(buffer, section[match.end():], tag)
+                return
 
-    scrolledwindow = textview.get_parent()
-
-    try:
-        va = scrolledwindow.get_vadjustment()
-    except AttributeError:
-        # scrolledwindow may have disappeared already while Nicotine+ was shutting down
-        return
-
-    bottom = (va.get_value() + va.get_page_size()) >= va.get_upper()
+        _append(buffer, section, tag)
 
+    line = str(line).strip("\n")
     buffer = textview.get_buffer()
-    text_iter_start, text_iter_end = buffer.get_bounds()
     linenr = buffer.get_line_count()
 
-    final_timestamp = None
-    ts = 0
+    if buffer.get_char_count() > 0:
+        _append(buffer, "\n", None)
 
-    if showstamp and NICOTINE.np.config.sections["logging"]["timestamps"]:
-        if timestamp_format and not timestamp:
-            final_timestamp = time.strftime(timestamp_format)
-            line = "%s %s" % (final_timestamp, line)
-        elif timestamp_format and timestamp:
-            final_timestamp = time.strftime(timestamp_format, time.localtime(timestamp))
-            line = "%s %s" % (final_timestamp, line)
+    if showstamp and timestamp_format and NICOTINE.np.config.sections["logging"]["timestamps"]:
+        if timestamp:
+            final_timestamp = time.strftime(timestamp_format, time.localtime(timestamp)) + " "
+        else:
+            final_timestamp = time.strftime(timestamp_format) + " "
 
-    # Ensure newlines are in the correct place
-    # We want them before the content, to prevent adding an empty line at the end of the TextView
-    line = line.strip("\n")
-    if text_iter_end.get_offset() > 0:
-        line = "\n" + line
+        _append(buffer, final_timestamp, tag)
 
-    if final_timestamp is not None:
-        ts = len("\n") + len(final_timestamp)
-
-    # Append timestamp, if one exists, cut it from remaining line (to avoid matching against username)
-    _append(buffer, line[:ts], tag)
-    line = line[ts:]
-
-    if find_urls:
+    if find_urls and NICOTINE.np.config.sections["urls"]["urlcatching"]:
         # Match first url
         match = URL_RE.search(line)
+
         # Highlight urls, if found and tag them
-        while NICOTINE.np.config.sections["urls"]["urlcatching"] and match:
-            start = line[:match.start()]
-            _usertag(buffer, start)
+        while match:
+            _usertag(buffer, line[:match.start()])
+
             url = match.group()
             urltag = _makeurltag(buffer, url)
-            line = line[match.end():]
 
             if url.startswith("slsk://") and NICOTINE.np.config.sections["urls"]["humanizeurls"]:
                 import urllib.parse
                 url = urllib.parse.unquote(url)
 
             _append(buffer, url, urltag)
+
             # Match remaining url
+            line = line[match.end():]
             match = URL_RE.search(line)
 
     if line:
         _usertag(buffer, line)
 
-    if scroll and bottom:
-        GLib.idle_add(scroll_bottom, scrolledwindow)
+    if scroll:
+        scrollable = textview.get_parent()
+        va = scrollable.get_vadjustment()
+
+        # Scroll to bottom if we had scrolled up less than ~2 lines previously
+        if (va.get_value() + va.get_page_size()) >= va.get_upper() - 40:
+            GLib.idle_add(scroll_bottom, scrollable)
 
     return linenr
 
@@ -980,8 +979,6 @@ class IconNotebook:
         if fulltext is None:
             fulltext = label
 
-        label_tab.set_tooltip_text(fulltext)
-
         # menu for all tabs
         label_tab_menu = ImageLabel(label)
         label_tab.connect('button_press_event', self.on_tab_click, page)
@@ -993,6 +990,8 @@ class IconNotebook:
 
         if status:
             self.set_user_status(page, label, status)
+        else:
+            label_tab.set_tooltip_text(fulltext)
 
         self.notebook.set_tab_reorderable(page, self.reorderable)
         self.notebook.set_show_tabs(True)
@@ -1004,6 +1003,24 @@ class IconNotebook:
         if self.notebook.get_n_pages() == 0:
             self.notebook.set_show_tabs(False)
 
+    def remove_all_pages(self):
+
+        option_dialog(
+            parent=NICOTINE.MainWindow,
+            title=_('Close All Tabs?'),
+            message=_('Are you sure you wish to close all tabs?'),
+            callback=self.remove_all_pages_cb
+        )
+
+    def remove_all_pages_cb(self, dialog, response, data):
+
+        if response == Gtk.ResponseType.OK:
+            for page in self.notebook.get_children():
+                tab_label, menu_label = self.get_labels(page)
+                tab_label.onclose(dialog)
+
+        dialog.destroy()
+
     def get_page_owner(self, page, items):
 
         n = self.page_num(page)
@@ -1015,9 +1032,18 @@ class IconNotebook:
         # Dummy implementation
         pass
 
-    def on_tab_click(self, widget, event, child):
-        # Dummy implementation
-        pass
+    def on_tab_click(self, widget, event, page):
+
+        if triggers_context_menu(event):
+            return self.on_tab_popup(widget, page)
+
+        elif event.button == 2:
+            # Middle click
+            tab_label, menu_label = self.get_labels(page)
+            tab_label.onclose(widget)
+            return True
+
+        return False
 
     def set_status_image(self, page, status):
 
@@ -1051,6 +1077,10 @@ class IconNotebook:
 
         self.set_status_image(page, status)
 
+        # Set a tab tooltip containing the user's status and name
+        tab_label, menu_label = self.get_labels(page)
+        tab_label.set_tooltip_text("%s (%s)" % (user, status_text))
+
     def set_hilite_image(self, page, status):
 
         tab_label, menu_label = self.get_labels(page)
@@ -1125,10 +1155,11 @@ class IconNotebook:
 
     def on_key_press_event(self, widget, event):
 
-        key = Gdk.keyval_name(event.keyval)
+        keycode = event.hardware_keycode
 
-        if event.state in (Gdk.ModifierType.CONTROL_MASK, Gdk.ModifierType.LOCK_MASK | Gdk.ModifierType.CONTROL_MASK):
-            if key in ("W", "w") or key == "F4":
+        if event.get_state() & Gdk.ModifierType.CONTROL_MASK:
+            if keycode in keyval_to_hardware_keycode(Gdk.KEY_w) or \
+               keycode in keyval_to_hardware_keycode(Gdk.KEY_F4):
                 # Ctrl+W and Ctrl+F4: close current tab
 
                 page = self.get_nth_page(self.get_current_page())
@@ -1314,10 +1345,10 @@ class PopupMenu(Gtk.Menu):
         self.editing = True
 
         self.items[_("_Add User To List")].set_active(self.user in (i[0] for i in self.frame.np.config.sections["server"]["userlist"]))
-        self.items[_("_Ban User")].set_active(self.user in self.frame.np.config.sections["server"]["banlist"])
-        self.items[_("_Ignore User")].set_active(self.user in self.frame.np.config.sections["server"]["ignorelist"])
-        self.items[_("B_lock User's IP Address")].set_active(self.frame.user_ip_is_blocked(self.user))
-        self.items[_("Ignore User's IP Address")].set_active(self.frame.user_ip_is_ignored(self.user))
+        self.items[_("_Ban User")].set_active(self.frame.np.network_filter.is_user_banned(self.user))
+        self.items[_("_Ignore User")].set_active(self.frame.np.network_filter.is_user_ignored(self.user))
+        self.items[_("B_lock User's IP Address")].set_active(self.frame.np.network_filter.get_cached_blocked_user_ip(self.user) or False)
+        self.items[_("Ignore User's IP Address")].set_active(self.frame.np.network_filter.get_cached_ignored_user_ip(self.user) or False)
 
         self.editing = False
 
@@ -1370,7 +1401,7 @@ class PopupMenu(Gtk.Menu):
     def on_show_ip_address(self, widget):
 
         self.frame.np.ip_requested.add(self.user)
-        self.frame.np.queue.put(slskmessages.GetPeerAddress(self.user))
+        self.frame.np.queue.append(slskmessages.GetPeerAddress(self.user))
 
     def on_get_user_info(self, widget):
         self.frame.local_user_info_request(self.user)
@@ -1379,16 +1410,16 @@ class PopupMenu(Gtk.Menu):
         self.frame.browse_user(self.user)
 
     def on_private_room_add_user(self, widget, room):
-        self.frame.np.queue.put(slskmessages.PrivateRoomAddUser(room, self.user))
+        self.frame.np.queue.append(slskmessages.PrivateRoomAddUser(room, self.user))
 
     def on_private_room_remove_user(self, widget, room):
-        self.frame.np.queue.put(slskmessages.PrivateRoomRemoveUser(room, self.user))
+        self.frame.np.queue.append(slskmessages.PrivateRoomRemoveUser(room, self.user))
 
     def on_private_room_add_operator(self, widget, room):
-        self.frame.np.queue.put(slskmessages.PrivateRoomAddOperator(room, self.user))
+        self.frame.np.queue.append(slskmessages.PrivateRoomAddOperator(room, self.user))
 
     def on_private_room_remove_operator(self, widget, room):
-        self.frame.np.queue.put(slskmessages.PrivateRoomRemoveOperator(room, self.user))
+        self.frame.np.queue.append(slskmessages.PrivateRoomRemoveOperator(room, self.user))
 
     def on_add_to_list(self, widget):
 
@@ -1406,9 +1437,9 @@ class PopupMenu(Gtk.Menu):
             return
 
         if widget.get_active():
-            self.frame.ban_user(self.user)
+            self.frame.np.network_filter.ban_user(self.user)
         else:
-            self.frame.unban_user(self.user)
+            self.frame.np.network_filter.unban_user(self.user)
 
     def on_block_user(self, widget):
 
@@ -1416,9 +1447,9 @@ class PopupMenu(Gtk.Menu):
             return
 
         if widget.get_active():
-            self.frame.on_block_user(self.user)
+            self.frame.np.network_filter.block_user_ip(self.user)
         else:
-            self.frame.on_un_block_user(self.user)
+            self.frame.np.network_filter.unblock_user_ip(self.user)
 
     def on_ignore_ip(self, widget):
 
@@ -1426,9 +1457,9 @@ class PopupMenu(Gtk.Menu):
             return
 
         if widget.get_active():
-            self.frame.on_ignore_ip(self.user)
+            self.frame.np.network_filter.ignore_user_ip(self.user)
         else:
-            self.frame.on_un_ignore_ip(self.user)
+            self.frame.np.network_filter.unignore_user_ip(self.user)
 
     def on_ignore_user(self, widget):
 
@@ -1436,9 +1467,9 @@ class PopupMenu(Gtk.Menu):
             return
 
         if widget.get_active():
-            self.frame.ignore_user(self.user)
+            self.frame.np.network_filter.ignore_user(self.user)
         else:
-            self.frame.unignore_user(self.user)
+            self.frame.np.network_filter.unignore_user(self.user)
 
     def on_version(self, widget):
         self.frame.privatechats.send_message(self.user, "\x01VERSION\x01", bytestring=True)
@@ -1448,7 +1479,7 @@ class PopupMenu(Gtk.Menu):
 
     def on_give_privileges(self, widget, error=None):
 
-        self.frame.np.queue.put(slskmessages.CheckPrivileges())
+        self.frame.np.queue.append(slskmessages.CheckPrivileges())
 
         if self.frame.np.privileges_left is None:
             days = _("Unknown")
@@ -1469,7 +1500,7 @@ class PopupMenu(Gtk.Menu):
         if days:
             try:
                 days = int(days)
-                self.frame.np.queue.put(slskmessages.GivePrivileges(self.user, days))
+                self.frame.np.queue.append(slskmessages.GivePrivileges(self.user, days))
 
             except ValueError:
                 self.on_give_privileges(widget, error=_("Please enter a whole number!"))
@@ -1504,15 +1535,6 @@ class PopupMenu(Gtk.Menu):
         popup.setup(*items)
         return True
 
-    def on_close_all_tabs(self, widget, caller):
-
-        option_dialog(
-            parent=self.frame.MainWindow,
-            title=_('Close All Tabs?'),
-            message=_('Are you sure you wish to close all tabs?'),
-            callback=caller.close_all_tabs
-        )
-
 
 class FileChooserButton:
     """ This class expands the functionality of a GtkButton to open a file
@@ -1663,10 +1685,9 @@ class TextSearchBar:
         self.on_search_match(search_type="next")
 
     def on_key_press(self, widget, event):
-        key = Gdk.keyval_name(event.keyval)
 
-        # Match against capslock + control and control
-        if key in ("f", "F") and event.state in (Gdk.ModifierType.CONTROL_MASK, Gdk.ModifierType.LOCK_MASK | Gdk.ModifierType.CONTROL_MASK):
+        if event.get_state() & Gdk.ModifierType.CONTROL_MASK and \
+                event.hardware_keycode in keyval_to_hardware_keycode(Gdk.KEY_f):
             self.show_search_bar()
 
     def show_search_bar(self):
@@ -1741,6 +1762,120 @@ def humanize(number):
     return neg + ret[:-1]
 
 
+def keyval_to_hardware_keycode(keyval):
+    """ This function is used for our custom keyboard shortcuts, such as Ctrl-C
+    in tree views, to ensure that they work on non-Latin keyboard layouts """
+
+    keymap = Gdk.Keymap.get_for_display(Gdk.Display.get_default())
+    valid, keymap_keys = keymap.get_entries_for_keyval(keyval)
+
+    if not valid:
+        return []
+
+    return [key.keycode for key in keymap_keys]
+
+
+""" Chat """
+
+
+def auto_replace(message):
+
+    config = NICOTINE.np.config.sections["words"]
+
+    if config["replacewords"]:
+        autoreplaced = config["autoreplaced"]
+
+        for word, replacement in autoreplaced.items():
+            message = message.replace(str(word), str(replacement))
+
+    return message
+
+
+def censor_chat(message):
+
+    config = NICOTINE.np.config.sections["words"]
+
+    if config["censorwords"]:
+        filler = config["censorfill"]
+        censored = config["censored"]
+
+        for word in censored:
+            word = str(word)
+            message = message.replace(word, filler * len(word))
+
+    return message
+
+
+def entry_completion_find_match(completion, entry_text, iterator):
+
+    entry = completion.get_entry()
+    model = completion.get_model()
+    item_text = model.get_value(iterator, 0)
+    ix = entry.get_position()
+    config = NICOTINE.np.config.sections["words"]
+
+    if entry_text is None or entry_text == "" or entry_text.isspace() or item_text is None:
+        return False
+
+    # Get word to the left of current position
+    if " " in entry_text:
+        split_key = entry_text[:ix].split(" ")[-1]
+    else:
+        split_key = entry_text
+
+    if split_key.isspace() or split_key == "" or len(split_key) < config["characters"]:
+        return False
+
+    # case-insensitive matching
+    if item_text.lower().startswith(split_key) and item_text.lower() != split_key:
+        return True
+
+    return False
+
+
+def entry_completion_found_match(completion, model, iterator):
+
+    entry = completion.get_entry()
+    current_text = entry.get_text()
+    ix = entry.get_position()
+    # if more than a word has been typed, we throw away the
+    # one to the left of our current position because we want
+    # to replace it with the matching word
+
+    if " " in current_text:
+        prefix = " ".join(current_text[:ix].split(" ")[:-1])
+        suffix = " ".join(current_text[ix:].split(" "))
+
+        # add the matching word
+        new_text = "%s %s%s" % (prefix, model[iterator][0], suffix)
+        # set back the whole text
+        entry.set_text(new_text)
+        # move the cursor at the end
+        entry.set_position(len(prefix) + len(model[iterator][0]) + 1)
+    else:
+        new_text = "%s" % (model[iterator][0])
+        entry.set_text(new_text)
+        entry.set_position(-1)
+
+    # stop the event propagation
+    return True
+
+
+def get_user_status_color(status):
+
+    if status == 1:
+        color = "useraway"
+    elif status == 2:
+        color = "useronline"
+    else:
+        color = "useroffline"
+
+    if not NICOTINE.np.config.sections["ui"]["showaway"] and color == "useraway":
+        color = "useronline"
+
+    return color
+
+
 """ Command Aliases """
 
 
@@ -1945,8 +2080,38 @@ def set_widget_font(widget, font):
     widget.set_property("font", font)
 
 
+def update_tag_visuals(tag, color):
+
+    config = NICOTINE.np.config.sections["ui"]
+
+    set_widget_color(tag, config[color])
+    set_widget_font(tag, config["chatfont"])
+
+    # Hotspots
+    if color in ("useraway", "useronline", "useroffline"):
+
+        usernamestyle = config["usernamestyle"]
+
+        if usernamestyle == "bold":
+            tag.set_property("weight", Pango.Weight.BOLD)
+        else:
+            tag.set_property("weight", Pango.Weight.NORMAL)
+
+        if usernamestyle == "italic":
+            tag.set_property("style", Pango.Style.ITALIC)
+        else:
+            tag.set_property("style", Pango.Style.NORMAL)
+
+        if usernamestyle == "underline":
+            tag.set_property("underline", Pango.Underline.SINGLE)
+        else:
+            tag.set_property("underline", Pango.Underline.NONE)
+
+
 def update_widget_visuals(widget, list_font_target="listfont", update_text_tags=True):
 
+    config = NICOTINE.np.config.sections["ui"]
+
     if isinstance(widget, Gtk.ComboBox) and widget.get_has_entry() or \
             isinstance(widget, Gtk.Entry):
         if isinstance(widget, Gtk.ComboBox):
@@ -1954,19 +2119,19 @@ def update_widget_visuals(widget, list_font_target="listfont", update_text_tags=
 
         set_widget_fg_bg_css(
             widget,
-            bg_color=NICOTINE.np.config.sections["ui"]["textbg"],
-            fg_color=NICOTINE.np.config.sections["ui"]["inputcolor"]
+            bg_color=config["textbg"],
+            fg_color=config["inputcolor"]
         )
 
     elif update_text_tags and isinstance(widget, Gtk.TextTag):
         # Chat rooms and private chats have their own code for this
 
-        set_widget_color(widget, NICOTINE.np.config.sections["ui"]["chatremote"])
-        set_widget_font(widget, NICOTINE.np.config.sections["ui"]["chatfont"])
+        set_widget_color(widget, config["chatremote"])
+        set_widget_font(widget, config["chatfont"])
 
     elif isinstance(widget, Gtk.TreeView):
-        set_list_color(widget, NICOTINE.np.config.sections["ui"]["search"])
-        set_list_font(widget, NICOTINE.np.config.sections["ui"][list_font_target])
+        set_list_color(widget, config["search"])
+        set_list_font(widget, config[list_font_target])
 
 
 """ Events """
