@@ -16,10 +16,9 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import os
 import time
 
-from pynicotine.utils import write_log
-
 
 class Logger(object):
 
@@ -69,13 +68,17 @@ class Logger(object):
         if level not in self.log_levels:
             return
 
+        if not msg_args and level == 4:
+            # Compile message contents
+            msg = "%s %s" % (msg.__class__, self.contents(msg))
+
         msg = self.set_msg_prefix(level, msg)
 
         if msg_args:
             msg = msg % msg_args
 
         if self.log_to_file:
-            write_log(self.folder, self.file_name, msg, self.timestamp_format)
+            self.write_log(self.folder, self.file_name, msg, self.timestamp_format)
 
         for callback in self.listeners:
             try:
@@ -101,6 +104,13 @@ class Logger(object):
     def add_debug(self, msg, msg_args=None):
         self.add(msg, msg_args=msg_args, level=6)
 
+    def contents(self, obj):
+        """ Returns variables for object, for debug output """
+        try:
+            return {s: getattr(obj, s) for s in obj.__slots__ if hasattr(obj, s)}
+        except AttributeError:
+            return vars(obj)
+
     def add_listener(self, callback):
         self.listeners.add(callback)
 
@@ -110,18 +120,41 @@ class Logger(object):
         except KeyError:
             self.add("Failed to remove listener %s, does not exist." % (callback,), 1)
 
-    def set_log_to_file(self, write):
-        self.log_to_file = write
-
-    def set_folder(self, folder):
-        self.folder = folder
-
-    def set_timestamp_format(self, timestamp_format):
-        self.timestamp_format = timestamp_format
-
     def set_log_levels(self, levels):
         self.log_levels = levels
 
+    def update_debug_log_options(self, should_log, log_folder, timestamp_format):
+        """ Gives the logger updated logging settings """
+
+        self.log_to_file = should_log
+        self.folder = log_folder
+        self.timestamp_format = timestamp_format
+
+    def write_log(self, logsdir, filename, msg, timestamp_format="%Y-%m-%d %H:%M:%S"):
+
+        try:
+            filename = filename.replace(os.sep, "-") + ".log"
+            oldumask = os.umask(0o077)
+
+            if not os.path.exists(logsdir):
+                os.makedirs(logsdir)
+
+            from pynicotine.utils import get_path
+            get_path(logsdir, filename, self.write_log_callback, (oldumask, timestamp_format, msg))
+
+        except Exception as error:
+            print(_("Couldn't write to log file \"%s\": %s") % (filename, error))
+
+    def write_log_callback(self, path, data):
+
+        oldumask, timestamp_format, msg = data
+
+        with open(path, 'ab', 0) as logfile:
+            os.umask(oldumask)
+
+            text = "%s %s\n" % (time.strftime(timestamp_format), msg)
+            logfile.write(text.encode('utf-8', 'replace'))
+
 
 class Console(object):
 
