@@ -22,7 +22,7 @@
 
 import os
 import sys
-import _thread
+import threading
 
 from ast import literal_eval
 from time import time
@@ -78,12 +78,17 @@ class PluginHandler(object):
         return None
 
     def toggle_plugin(self, pluginname):
-        on = pluginname in self.enabled_plugins
 
-        if on:
-            _thread.start_new_thread(self.disable_plugin, (pluginname,))
+        enabled = pluginname in self.enabled_plugins
+
+        if enabled:
+            thread = threading.Thread(target=self.disable_plugin, args=(pluginname,))
         else:
-            _thread.start_new_thread(self.enable_plugin, (pluginname,))
+            thread = threading.Thread(target=self.enable_plugin, args=(pluginname,))
+
+        thread.name = "TogglePlugin"
+        thread.daemon = True
+        thread.start()
 
     def load_plugin(self, pluginname):
 
@@ -100,8 +105,10 @@ class PluginHandler(object):
                 log.add(_("Failed to load plugin '%s', could not find it."), pluginname)
                 return False
 
-            from importlib.machinery import SourceFileLoader
-            plugin = SourceFileLoader(pluginname, os.path.join(path, '__init__.py')).load_module()
+            import importlib.util
+            spec = importlib.util.spec_from_file_location(pluginname, os.path.join(path, '__init__.py'))
+            plugin = importlib.util.module_from_spec(spec)
+            spec.loader.exec_module(plugin)
 
         instance = plugin.Plugin(self, enable_plugin=False)
         self.plugin_settings(pluginname, instance)
@@ -114,7 +121,10 @@ class PluginHandler(object):
 
         # Our config file doesn't play nicely with some characters
         if "=" in pluginname:
-            log.add(_("Unable to enable plugin %(name)s. Plugin folder name contains invalid characters: %(characters)s") % {"name": pluginname, "characters": "="})
+            log.add(_("Unable to enable plugin %(name)s. Plugin folder name contains invalid characters: %(characters)s"), {
+                "name": pluginname,
+                "characters": "="
+            })
             return
 
         if pluginname in self.enabled_plugins:
@@ -202,7 +212,10 @@ class PluginHandler(object):
         to_enable = self.config.sections["plugins"]["enabled"]
 
         for plugin in to_enable:
-            _thread.start_new_thread(self.enable_plugin, (plugin,))
+            thread = threading.Thread(target=self.enable_plugin, args=(plugin,))
+            thread.name = "EnablePlugin"
+            thread.daemon = True
+            thread.start()
 
     def plugin_settings(self, pluginname, plugin):
         try:
@@ -253,9 +266,9 @@ class PluginHandler(object):
                 if return_value == returncode['zap']:
                     return True
                 elif return_value == returncode['pass']:
-                    pass
-                else:
-                    log.add(_("Plugin %(module)s returned something weird, '%(value)s', ignoring"), {'module': module, 'value': str(return_value)})
+                    continue
+
+                log.add(_("Plugin %(module)s returned something weird, '%(value)s', ignoring"), {'module': module, 'value': str(return_value)})
 
             except Exception:
                 from traceback import extract_stack
@@ -295,9 +308,9 @@ class PluginHandler(object):
                 elif return_value == returncode['break']:
                     return args
                 elif return_value == returncode['pass']:
-                    pass
-                else:
-                    log.add(_("Plugin %(module)s returned something weird, '%(value)s', ignoring"), {'module': module, 'value': return_value})
+                    continue
+
+                log.add(_("Plugin %(module)s returned something weird, '%(value)s', ignoring"), {'module': module, 'value': return_value})
 
             except Exception:
                 from traceback import extract_stack
@@ -406,7 +419,7 @@ class PluginHandler(object):
         log.add(text)
 
     def saychatroom(self, room, text):
-        self.frame.np.queue.put(slskmessages.SayChatroom(room, text))
+        self.frame.np.queue.append(slskmessages.SayChatroom(room, text))
 
     def sayprivate(self, user, text):
         '''Send user message in private (showing up in GUI)'''
@@ -458,10 +471,10 @@ class ResponseThrottle(object):
         except Exception:
             port = True
 
-        if nick in self.frame.np.config.sections["server"]["ignorelist"]:
+        if self.frame.np.network_filter.is_user_ignored(nick):
             willing_to_respond, reason = False, "The nick is ignored"
 
-        elif self.frame.user_ip_is_ignored(nick):
+        elif self.frame.np.network_filter.is_user_ip_ignored(nick):
             willing_to_respond, reason = False, "The nick's Ip is ignored"
 
         elif not port:
