@@ -24,12 +24,11 @@ import importlib
 import multiprocessing
 import os
 import pickle
-import re
 import shelve
 import stat
 import string
 import sys
-import _thread
+import threading
 import time
 
 from pynicotine import slskmessages
@@ -494,6 +493,9 @@ class Shares:
         self.connected = connected
         self.translatepunctuation = str.maketrans(dict.fromkeys(string.punctuation, ' '))
         self.share_dbs = {}
+        self.public_rescanning = False
+        self.buddy_rescanning = False
+        self.initiated_shares = False
 
         self.convert_shares()
         self.public_share_dbs = [
@@ -510,11 +512,15 @@ class Shares:
             ("buddyfileindex", os.path.join(self.config.data_dir, "buddyfileindex.db")),
             ("buddymtimes", os.path.join(self.config.data_dir, "buddymtimes.db"))
         ]
-        self.load_shares(self.share_dbs, self.public_share_dbs)
-        self.load_shares(self.share_dbs, self.buddy_share_dbs)
 
-        self.create_compressed_shares_message("normal")
-        self.create_compressed_shares_message("buddy")
+        if ui_callback:
+            # Slight delay to prevent minor performance hit when compressing large file share
+            timer = threading.Timer(0.75, self.init_shares)
+            timer.name = "InitSharesTimer"
+            timer.daemon = True
+            timer.start()
+        else:
+            self.init_shares()
 
         self.newbuddyshares = self.newnormalshares = False
 
@@ -578,6 +584,32 @@ class Shares:
 
         return mapping
 
+    def init_shares(self):
+
+        self.load_shares(self.share_dbs, self.public_share_dbs)
+        self.load_shares(self.share_dbs, self.buddy_share_dbs)
+
+        self.create_compressed_shares_message("normal")
+        self.create_compressed_shares_message("buddy")
+
+        rescan_startup = self.config.sections["transfers"]["rescanonstartup"]
+
+        # Rescan public shares if necessary
+        if not self.config.sections["transfers"]["friendsonly"]:
+            if rescan_startup and not self.public_rescanning:
+                self.rescan_public_shares()
+            else:
+                self.compress_shares("normal")
+
+        # Rescan buddy shares if necessary
+        if self.config.sections["transfers"]["enablebuddyshares"]:
+            if rescan_startup and not self.buddy_rescanning:
+                self.rescan_buddy_shares()
+            else:
+                self.compress_shares("buddy")
+
+        self.initiated_shares = True
+
     def convert_shares(self):
         """ Convert fs-based shared to virtual shared (pre 1.4.0) """
 
@@ -585,20 +617,24 @@ class Shares:
             if isinstance(x, tuple):
                 return x
             virtual = x.replace('/', '_').replace('\\', '_').strip('_')
-            log.add("Renaming shared folder '%s' to '%s'. A rescan of your share is required." % (x, virtual))
+            log.add("Renaming shared folder '%s' to '%s'. A rescan of your share is required.", (x, virtual))
             return (virtual, x)
 
         self.config.sections["transfers"]["shared"] = [_convert_to_virtual(x) for x in self.config.sections["transfers"]["shared"]]
         self.config.sections["transfers"]["buddyshared"] = [_convert_to_virtual(x) for x in self.config.sections["transfers"]["buddyshared"]]
 
     @classmethod
-    def load_shares(cls, shares, dbs):
+    def load_shares(cls, shares, dbs, reset_shares=False):
 
         errors = []
 
         for destination, shelvefile in dbs:
             try:
-                shares[destination] = shelve.open(shelvefile, protocol=pickle.HIGHEST_PROTOCOL)
+                if not reset_shares:
+                    shares[destination] = shelve.open(shelvefile, protocol=pickle.HIGHEST_PROTOCOL)
+                else:
+                    shares[destination] = shelve.open(shelvefile, flag='n', protocol=pickle.HIGHEST_PROTOCOL)
+
             except Exception:
                 from traceback import format_exc
 
@@ -607,11 +643,15 @@ class Shares:
                 exception = format_exc()
 
         if errors:
-            log.add_warning(_("Failed to process the following databases: %(names)s") % {'names': '\n'.join(errors)})
+            log.add_warning(_("Failed to process the following databases: %(names)s"), {
+                'names': '\n'.join(errors)
+            })
             log.add_warning(exception)
-
             log.add_warning(_("Shared files database seems to be corrupted, rescan your shares"))
 
+            if not reset_shares:
+                cls.load_shares(shares, dbs, reset_shares=True)
+
     def add_file_to_shared(self, name):
         """ Add a file to the normal shares database """
 
@@ -619,6 +659,9 @@ class Shares:
         if not config["transfers"]["sharedownloaddir"]:
             return
 
+        if not self.initiated_shares or self.public_rescanning:
+            return
+
         shared = self.share_dbs["files"]
         sharedstreams = self.share_dbs["streams"]
         wordindex = self.share_dbs["wordindex"]
@@ -662,6 +705,9 @@ class Shares:
         if not config["transfers"]["sharedownloaddir"]:
             return
 
+        if not self.initiated_shares or self.buddy_rescanning:
+            return
+
         bshared = self.share_dbs["buddyfiles"]
         bsharedstreams = self.share_dbs["buddystreams"]
         bwordindex = self.share_dbs["buddywordindex"]
@@ -718,10 +764,14 @@ class Shares:
         quickly send our file list to users. """
 
         if sharestype == "normal":
-            _thread.start_new_thread(self.compressed_shares_normal.make_network_message, (0, True))
+            thread = threading.Thread(target=self.compressed_shares_normal.make_network_message, args=(0, True))
 
         elif sharestype == "buddy":
-            _thread.start_new_thread(self.compressed_shares_buddy.make_network_message, (0, True))
+            thread = threading.Thread(target=self.compressed_shares_buddy.make_network_message, args=(0, True))
+
+        thread.name = "CompressShares"
+        thread.daemon = True
+        thread.start()
 
     def close_shares(self, sharestype):
 
@@ -750,7 +800,7 @@ class Shares:
         if config["transfers"]["enablebuddyshares"] and config["transfers"]["friendsonly"]:
             # No public shares
             files = folders = 0
-            self.queue.put(slskmessages.SharedFoldersFiles(files, folders))
+            self.queue.append(slskmessages.SharedFoldersFiles(files, folders))
             return
 
         shared_db = "files"
@@ -765,7 +815,7 @@ class Shares:
                 sharedfolders = len(list(self.share_dbs[shared_db]))
                 sharedfiles = len(list(self.share_dbs[index_db]))
 
-            self.queue.put(slskmessages.SharedFoldersFiles(sharedfolders, sharedfiles))
+            self.queue.append(slskmessages.SharedFoldersFiles(sharedfolders, sharedfiles))
 
         except Exception as e:
             log.add(_("Failed to send number of shared files to the server: %s"), e)
@@ -786,13 +836,13 @@ class Shares:
         return scanner, scanner_queue
 
     def rebuild_public_shares(self, thread=True):
-        self.rescan_shares("normal", rebuild=True, thread=thread)
+        self.rescan_shares("normal", rebuild=True, use_thread=thread)
 
     def rescan_public_shares(self, rebuild=False, thread=True):
         self.rescan_shares("normal", rebuild, thread)
 
     def rebuild_buddy_shares(self, thread=True):
-        self.rescan_shares("buddy", rebuild=True, thread=thread)
+        self.rescan_shares("buddy", rebuild=True, use_thread=thread)
 
     def rescan_buddy_shares(self, rebuild=False, thread=True):
         self.rescan_shares("buddy", rebuild, thread)
@@ -800,11 +850,9 @@ class Shares:
     def get_shared_folders(self, sharestype):
 
         if sharestype == "normal":
-            log.add(_("Rescanning normal shares..."))
             shared_folders = self.config.sections["transfers"]["shared"][:]
 
         else:
-            log.add(_("Rescanning buddy shares..."))
             shared_folders = self.config.sections["transfers"]["buddyshared"][:] + self.config.sections["transfers"]["shared"][:]
 
         if self.config.sections["transfers"]["sharedownloaddir"]:
@@ -836,7 +884,21 @@ class Shares:
 
         return False
 
-    def rescan_shares(self, sharestype, rebuild=False, thread=True):
+    def rescan_shares(self, sharestype, rebuild=False, use_thread=True):
+
+        if sharestype == "normal" and self.public_rescanning:
+            return
+
+        if sharestype == "buddy" and self.buddy_rescanning:
+            return
+
+        if sharestype == "normal":
+            log.add(_("Rescanning normal shares..."))
+            self.public_rescanning = True
+
+        else:
+            log.add(_("Rescanning buddy shares..."))
+            self.buddy_rescanning = True
 
         shared_folders = self.get_shared_folders(sharestype)
 
@@ -846,8 +908,11 @@ class Shares:
         scanner, scanner_queue = self.build_scanner_process(shared_folders, sharestype, rebuild)
         scanner.start()
 
-        if thread:
-            _thread.start_new_thread(self._process_scanner, (scanner, scanner_queue, sharestype))
+        if use_thread:
+            thread = threading.Thread(target=self._process_scanner, args=(scanner, scanner_queue, sharestype))
+            thread.name = "ProcessShareScanner"
+            thread.daemon = True
+            thread.start()
         else:
             self._process_scanner(scanner, scanner_queue, sharestype)
 
@@ -862,8 +927,10 @@ class Shares:
 
         # Scanning done, load shares in the main process again
         if sharestype == "normal":
+            self.public_rescanning = False
             self.load_shares(self.share_dbs, self.public_share_dbs)
         else:
+            self.buddy_rescanning = False
             self.load_shares(self.share_dbs, self.buddy_share_dbs)
 
         self.create_compressed_shares_message(sharestype)
@@ -882,126 +949,4 @@ class Shares:
                 log.add(_("Finished rescanning buddy shares"))
 
         if self.ui_callback:
-            self.ui_callback.rescan_finished(sharestype)
-
-    """ Search request processing """
-
-    def create_search_result_list(self, searchterm, wordindex, maxresults=50):
-
-        try:
-            """ Stage 1: Check if each word in the search term is included in our word index.
-            If this is the case, we select the word that has the most file matches in our
-            word index. If not, exit, since we don't have relevant results. """
-
-            largest = 0
-
-            for i in re.finditer(r'\S+', searchterm):
-                i = i.group(0)
-
-                if i not in wordindex:
-                    return
-
-                list_size = len(wordindex[i])
-
-                if list_size > largest:
-                    largest = list_size
-                    largest_key = i
-
-            """ Stage 2: Start with the word that has the most file matches, which we selected
-            in the previous step, and gradually remove matches that other words in the search
-            term don't have. Return the remaining matches, if any. """
-
-            results = wordindex[largest_key]
-            searchterm.replace(largest_key, '')
-
-            for i in re.finditer(r'\S+', searchterm):
-                results = set(results).intersection(wordindex[i.group(0)])
-
-            return results
-
-        except ValueError:
-            # DB is closed, perhaps when rescanning share or closing Nicotine+
-            return
-
-    def process_search_request(self, searchterm, user, searchid, direct=False):
-        """ Note: since this section is accessed every time a search request arrives,
-        several times a second, please keep it as optimized and memory
-        sparse as possible! """
-
-        if not self.config.sections["searches"]["search_results"]:
-            # Don't return _any_ results when this option is disabled
-            return
-
-        if searchterm is None:
-            return
-
-        if not direct and user == self.config.sections["server"]["login"]:
-            # We shouldn't send a search response if we initiated the search request,
-            # unless we're specifically searching our own username
-            return
-
-        maxresults = self.config.sections["searches"]["maxresults"]
-
-        if maxresults == 0:
-            return
-
-        # Don't count excluded words as matches (words starting with -)
-        # Strip punctuation
-        searchterm = re.sub(r'(\s)-\w+', r'\1', searchterm).lower().translate(self.translatepunctuation).strip()
-
-        if len(searchterm) < self.config.sections["searches"]["min_search_chars"]:
-            # Don't send search response if search term contains too few characters
-            return
-
-        checkuser, reason = self.np.check_user(user, None)
-
-        if not checkuser:
-            return
-
-        if checkuser == 2:
-            wordindex = self.share_dbs["buddywordindex"]
-        else:
-            wordindex = self.share_dbs["wordindex"]
-
-        # Find common file matches for each word in search term
-        resultlist = self.create_search_result_list(searchterm, wordindex, maxresults)
-
-        if not resultlist:
-            return
-
-        if self.np.transfers is not None:
-
-            numresults = min(len(resultlist), maxresults)
-            queuesizes = self.np.transfers.get_upload_queue_sizes()
-            slotsavail = self.np.transfers.allow_new_uploads()
-
-            if checkuser == 2:
-                fileindex = self.share_dbs["buddyfileindex"]
-            else:
-                fileindex = self.share_dbs["fileindex"]
-
-            fifoqueue = self.config.sections["transfers"]["fifoqueue"]
-
-            message = slskmessages.FileSearchResult(
-                None,
-                self.config.sections["server"]["login"],
-                searchid, resultlist, fileindex, slotsavail,
-                self.np.speed, queuesizes, fifoqueue, numresults
-            )
-
-            self.np.send_message_to_peer(user, message)
-
-            if direct:
-                log.add_search(
-                    _("User %(user)s is directly searching for \"%(query)s\", returning %(num)i results"), {
-                        'user': user,
-                        'query': searchterm,
-                        'num': numresults
-                    })
-            else:
-                log.add_search(
-                    _("User %(user)s is searching for \"%(query)s\", returning %(num)i results"), {
-                        'user': user,
-                        'query': searchterm,
-                        'num': numresults
-                    })
+            self.ui_callback.hide_scan_progress(sharestype)
