@@ -121,7 +121,7 @@ from pynicotine.slskmessages import PrivateRoomUsers
 from pynicotine.slskmessages import PrivilegedUsers
 from pynicotine.slskmessages import PublicRoomMessage
 from pynicotine.slskmessages import QueuedDownloads
-from pynicotine.slskmessages import QueueFailed
+from pynicotine.slskmessages import UploadDenied
 from pynicotine.slskmessages import QueueUpload
 from pynicotine.slskmessages import Recommendations
 from pynicotine.slskmessages import RelatedSearch
@@ -199,7 +199,7 @@ else:
     try:
         resource.setrlimit(resource.RLIMIT_NOFILE, (maxfilelimit, maxfilelimit))
     except Exception as e:
-        log.add_warning("Failed to set RLIMIT_NOFILE: %s" % e)
+        log.add_warning("Failed to set RLIMIT_NOFILE: %s", e)
 
     """ Set the maximum number of open sockets to a lower value than the hard limit,
     otherwise we just waste resources.
@@ -341,9 +341,9 @@ class SlskProtoThread(threading.Thread):
         GivePrivileges: 123,
         NotifyPrivileges: 124,
         AckNotifyPrivileges: 125,
-        BranchLevel: 126,             # Unimplemented
-        BranchRoot: 127,              # Unimplemented
-        ChildDepth: 129,              # Unimplemented
+        BranchLevel: 126,
+        BranchRoot: 127,
+        ChildDepth: 129,
         PrivateRoomUsers: 133,
         PrivateRoomAddUser: 134,
         PrivateRoomRemoveUser: 135,
@@ -383,7 +383,7 @@ class SlskProtoThread(threading.Thread):
         QueueUpload: 43,
         PlaceInQueue: 44,
         UploadFailed: 46,
-        QueueFailed: 50,
+        UploadDenied: 50,
         PlaceInQueueRequest: 51,
         UploadQueueNotification: 52,
         UnknownPeerMessage: 12547
@@ -392,8 +392,8 @@ class SlskProtoThread(threading.Thread):
     distribclasses = {
         0: DistribAlive,
         3: DistribSearch,
-        4: DistribBranchLevel,        # Unimplemented
-        5: DistribBranchRoot,         # Unimplemented
+        4: DistribBranchLevel,
+        5: DistribBranchRoot,
         7: DistribChildDepth,         # Unimplemented
         93: DistribServerSearch
     }
@@ -402,7 +402,7 @@ class SlskProtoThread(threading.Thread):
     CONNECTION_MAX_IDLE = 60
     CONNCOUNT_UI_INTERVAL = 0.5
 
-    def __init__(self, ui_callback, queue, bindip, port, config, eventprocessor):
+    def __init__(self, ui_callback, queue, bindip, port, port_range, network_filter, eventprocessor):
         """ ui_callback is a UI callback function to be called with messages
         list as a parameter. queue is Queue object that holds messages from UI
         thread.
@@ -416,7 +416,7 @@ class SlskProtoThread(threading.Thread):
         self._want_abort = False
         self._server_disconnect = True
         self._bindip = bindip
-        self._config = config
+        self._network_filter = network_filter
         self._eventprocessor = eventprocessor
 
         self.serverclasses = {}
@@ -441,7 +441,7 @@ class SlskProtoThread(threading.Thread):
 
         self.last_conncount_ui_update = time.time()
 
-        portrange = (port, port) if port else config.sections["server"]["portrange"]
+        portrange = (port, port) if port else port_range
         listenport = None
 
         for listenport in range(int(portrange[0]), int(portrange[1]) + 1):
@@ -455,7 +455,7 @@ class SlskProtoThread(threading.Thread):
                 break
 
         if listenport is not None:
-            self.setDaemon(True)
+            self.daemon = True
             self.start()
         else:
             short_message = _("Could not bind to a local port, aborting connection")
@@ -549,39 +549,6 @@ class SlskProtoThread(threading.Thread):
 
         return len(connection.obuf) > 0 or len(connection.ibuf) > 0
 
-    def ip_blocked(self, address):
-        if address is None:
-            return True
-
-        ips = self._config.sections["server"]["ipblocklist"]
-        s_address = address.split(".")
-
-        for ip in ips:
-            # No Wildcard in IP
-            if "*" not in ip:
-                if address == ip:
-                    return True
-                continue
-
-            # Wildcard in IP
-            parts = ip.split(".")
-            seg = 0
-
-            for part in parts:
-                # Stop if there's no wildcard or matching string number
-                if part != s_address[seg] and part != "*":
-                    break
-
-                seg += 1
-
-                # Last time around
-                if seg == 4:
-                    # Wildcard blocked
-                    return True
-
-        # Not blocked
-        return False
-
     def parse_file_req(self, conn, msg_buffer):
         msg = None
 
@@ -933,8 +900,8 @@ class SlskProtoThread(threading.Thread):
         needsleep = False
         numsockets = len(conns) + len(connsinprogress)
 
-        while not queue.empty():
-            msg_list.append(queue.get())
+        while queue:
+            msg_list.append(queue.popleft())
 
         for msg_obj in msg_list:
             if issubclass(msg_obj.__class__, ServerMessage):
@@ -945,7 +912,7 @@ class SlskProtoThread(threading.Thread):
                         conns[server_socket].obuf.extend(struct.pack("<ii", len(msg) + 4, self.servercodes[msg_obj.__class__]))
                         conns[server_socket].obuf.extend(msg)
                     else:
-                        queue.put(msg_obj)
+                        queue.append(msg_obj)
                         needsleep = True
 
                 except Exception as error:
@@ -989,7 +956,10 @@ class SlskProtoThread(threading.Thread):
 
                 else:
                     if msg_obj.__class__ not in [PeerInit, PierceFireWall, FileSearchResult]:
-                        log.add_conn("Can't send the message over the closed connection: %(type)s %(msg_obj)s", {'type': msg_obj.__class__, 'msg_obj': vars(msg_obj)})
+                        log.add_conn("Can't send the message over the closed connection: %(type)s %(msg_obj)s", {
+                            'type': msg_obj.__class__,
+                            'msg_obj': vars(msg_obj)
+                        })
 
             elif issubclass(msg_obj.__class__, InternalMessage):
                 if msg_obj.__class__ is ServerConn:
@@ -1052,7 +1022,8 @@ class SlskProtoThread(threading.Thread):
                             self._ui_callback([ConnectError(msg_obj, err)])
                             conn.close()
                     else:
-                        self._ui_callback([ConnectError(msg_obj, "Connection limit reached")])
+                        # Connection limit reached, re-queue
+                        queue.append(msg_obj)
 
                 elif msg_obj.__class__ is DownloadFile and msg_obj.conn in conns:
                     conns[msg_obj.conn].filedown = msg_obj
@@ -1203,7 +1174,7 @@ class SlskProtoThread(threading.Thread):
                 time.sleep(0.2)
                 continue
 
-            if not queue.empty():
+            if queue:
                 conns, connsinprogress, server_socket = self.process_queue(queue, conns, connsinprogress, server_socket)
                 self._server_socket = server_socket
 
@@ -1220,7 +1191,7 @@ class SlskProtoThread(threading.Thread):
                     conn = conns[i]
                     event_masks = selectors.EVENT_READ
 
-                    if len(conn.obuf) > 0 or (i is not server_socket and conn.fileupl is not None and conn.fileupl.offset is not None):
+                    if conn.obuf or (i is not server_socket and conn.fileupl is not None and conn.fileupl.offset is not None):
                         if self._is_upload(conn):
                             limit = self._uploadlimit[0](conns, conn)
 
@@ -1281,7 +1252,7 @@ class SlskProtoThread(threading.Thread):
                 except Exception:
                     time.sleep(0.01)
                 else:
-                    if self.ip_blocked(incaddr[0]):
+                    if self._network_filter.is_ip_blocked(incaddr[0]):
                         log.add_conn(_("Ignoring connection request from blocked IP Address %(ip)s:%(port)s"), {
                             'ip': incaddr[0],
                             'port': incaddr[1]
@@ -1324,12 +1295,8 @@ class SlskProtoThread(threading.Thread):
                             self._ui_callback([ServerConn(server_socket, addr)])
 
                         else:
-                            if self.ip_blocked(addr[0]):
-                                log.add_conn("Blocking peer connection in progress to IP: %(ip)s Port: %(port)s", {"ip": addr[0], "port": addr[1]})
-                                connection_in_progress.close()
-                            else:
-                                conns[connection_in_progress] = PeerConnection(conn=connection_in_progress, addr=addr, init=msg_obj.init)
-                                self._ui_callback([OutConn(connection_in_progress, addr)])
+                            conns[connection_in_progress] = PeerConnection(conn=connection_in_progress, addr=addr, init=msg_obj.init)
+                            self._ui_callback([OutConn(connection_in_progress, addr)])
 
                         del connsinprogress[connection_in_progress]
 
@@ -1361,8 +1328,11 @@ class SlskProtoThread(threading.Thread):
                             self.close_connection(conns, connection)
                             continue
 
-                    if self.ip_blocked(addr[0]):
-                        log.add_conn("Blocking peer connection to IP: %(ip)s Port: %(port)s", {"ip": addr[0], "port": addr[1]})
+                    if self._network_filter.is_ip_blocked(addr[0]):
+                        log.add_conn("Blocking peer connection to IP: %(ip)s Port: %(port)s", {
+                            "ip": addr[0],
+                            "port": addr[1]
+                        })
                         self.close_connection(conns, connection)
                         continue
 
@@ -1385,7 +1355,7 @@ class SlskProtoThread(threading.Thread):
                         continue
 
                 try:
-                    if len(conn_obj.ibuf) > 0:
+                    if conn_obj.ibuf:
                         if connection is server_socket:
                             msgs, conns[server_socket].ibuf = self.process_server_input(conns[server_socket].ibuf)
                             self._ui_callback(msgs)
@@ -1430,8 +1400,8 @@ class SlskProtoThread(threading.Thread):
             i.close()
             del self._connsinprogress[i]
 
-        while not self._queue.empty():
-            self._queue.get(0)
+        while self._queue:
+            self._queue.popleft()
 
         if not self._want_abort:
             self._ui_callback([SetCurrentConnectionCount(0)])
